from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset, ItemRevision, ImanRelation, Item, User, BusinessObject, Folder, Group
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExtendedAttributes(TcBaseObj):
    """
    This structure contains information for 'createItems' operation to support setting attribute values on the created
    Item, ItemRevision, or corresponding master forms that may be created with the objects.
    
    :var objectType: The type of object to set these properties on i.e. Connection, ConnectionRevision
    :var attributes: A map of  attributes names and initial values pairs ('string/string'). Multi-valued properties are
    represented with a comma separated string
    """
    objectType: str = ''
    attributes: AttributeNameValueMap = None


@dataclass
class GenerateItemIdsAndInitialRevisionIdsProperties(TcBaseObj):
    """
    The input information for 'generateItemIdsAndInitialRevisionIds' operation.
    
    :var item: Item object reference to use as a basis for the next Item ID. This value is optional.
    :var itemType: Type of the Item for which the IDs will be generated.  This value is optional.  The default is Item.
    :var count: Total number of IDs to be generated
    """
    item: BusinessObject = None
    itemType: str = ''
    count: int = 0


@dataclass
class GenerateItemIdsAndInitialRevisionIdsResponse(TcBaseObj):
    """
    Return structure for 'generateItemIdsAndInitialRevisionIds' operation
    
    :var outputItemIdsAndInitialRevisionIds: A list of the new Item and ItemRevision IDs and flags indicating if the
    system is configured to allow modification of the id values after they have been generated.
    :var serviceData: Service data
    """
    outputItemIdsAndInitialRevisionIds: IndexToIdMap = None
    serviceData: ServiceData = None


@dataclass
class GenerateRevisionIdsProperties(TcBaseObj):
    """
    The data structure contains information for 'generateRevisionIds' operation
    
    :var item: Item object reference to get the next revision id, optional
    :var itemType: Type of the ids to generate, optional
    """
    item: BusinessObject = None
    itemType: str = ''


@dataclass
class GenerateRevisionIdsResponse(TcBaseObj):
    """
    The data structure contains returned information for 'generateRevisionIds' operation
    
    :var outputRevisionIds: A list of the new generated id values
    :var serviceData: Service data
    """
    outputRevisionIds: IndexToRevIdMap = None
    serviceData: ServiceData = None


@dataclass
class ItemIdsAndInitialRevisionIds(TcBaseObj):
    """
    This structure contain output information for 'generateItemIdsAndInitialRevisionIds' operation indicating the new
    item id, new revision id, and a flag for each that indicates if the caller should be able to override the generated
    values.
    
    :var newItemId: Newly generated Item Id
    :var newRevId: Newly generated ItemRevision Id
    :var isItemModify: Flag to specify whether Item id is modifiable
    :var isRevModify: Flag to specify whether ItemRevision id is modifiable
    """
    newItemId: str = ''
    newRevId: str = ''
    isItemModify: bool = False
    isRevModify: bool = False


@dataclass
class ItemProperties(TcBaseObj):
    """
    Input structure for createItems operation.  Specifies attributes for the new Item or ItemRevision.
    
    :var clientId: Identifier that helps the client to track the object(s) created, optional.
    :var itemId: Id of the Item to be created, optional
    :var name: Name of the Item to be created, optional
    :var type: Type of the Item to be created, optional, default is Item
    :var revId: Id of the initail revision of the Item to be created, optional
    :var uom: Unit of measure(UOM) of the Item to be created, optional
    :var description: Description of the Item to be created, optional
    :var extendedAttributes: Name/value pairs for additional attributes to set, optional
    """
    clientId: str = ''
    itemId: str = ''
    name: str = ''
    type: str = ''
    revId: str = ''
    uom: str = ''
    description: str = ''
    extendedAttributes: List[ExtendedAttributes] = ()


@dataclass
class ObjectOwner(TcBaseObj):
    """
    This structure contains the business object whose owner needs to be changed, new owning user of the object and new
    owning group of the object.
    
    :var object: Teamcenter Business object.
    :var owner: New owning user of the business object.
    :var group: New owning group of the business object.
    """
    object: BusinessObject = None
    owner: User = None
    group: Group = None


@dataclass
class Relationship(TcBaseObj):
    """
    'Relationship' structure represents all required parameters to create the relation between the primary and
    secondary object.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this 'Relationship' structure.
    :var relationType: Name of the relation type to create, required. This could be an empty string, in which case the
    relation name will be searched in the preference, ParentTypeName_ChildTypeName_default_relation or
    ParentTypeName_default_relation.
    :var primaryObject: The primary object to create the relation from
    :var secondaryObject: The secondary object to create the relation to.
    :var userData: The user data object used to create the relation. This parameter is optional.
    """
    clientId: str = ''
    relationType: str = ''
    primaryObject: BusinessObject = None
    secondaryObject: BusinessObject = None
    userData: BusinessObject = None


@dataclass
class ReviseProperties(TcBaseObj):
    """
    This structure contains information about new revision id, name, description and extended attributes.
    
    :var revId: New revision id of the Item to be revised, optional
    :var name: Name of the new ItemRevision, optional
    :var description: Description of the new ItemRevision, optional
    :var extendedAttributes: Name/value pairs for additional attributes to set, optional
    """
    revId: str = ''
    name: str = ''
    description: str = ''
    extendedAttributes: AttributeNameValueMap = None


@dataclass
class ReviseResponse(TcBaseObj):
    """
    Return structure for revise operation
    
    :var oldItemRevToNewItemRev: A Map whose keys are the input old item revisions and values are the newly created
    revisions
    :var serviceData: Standard ServiceData member
    """
    oldItemRevToNewItemRev: ItemRevMap = None
    serviceData: ServiceData = None


@dataclass
class RevisionIds(TcBaseObj):
    """
    This structure contains information for 'generateRevisionIds' operation
    
    :var newRevId: Revision id that was generated
    :var isModify: Flag to specify whether revision id is modifiable
    """
    newRevId: str = ''
    isModify: bool = False


@dataclass
class CreateDatasetsOutput(TcBaseObj):
    """
    This structure contains information for createDatasets operation.
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var dataset: Dataset object reference that was created
    """
    clientId: str = ''
    dataset: Dataset = None


@dataclass
class CreateDatasetsResponse(TcBaseObj):
    """
    Return structure for createDatasets operation
    
    :var output: A list of created Dataset output
    :var serviceData: Standard 'ServiceData' member
    """
    output: List[CreateDatasetsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateFolderInput(TcBaseObj):
    """
    Input structure for createFolders operation
    
    :var clientId: Identifier that helps the client to track the object created, optional.
    :var name: Name of the folder to be created, optional, if null, uses value from USER_new_folder_name user exit
    :var desc: Description of the folder to be created, optional
    """
    clientId: str = ''
    name: str = ''
    desc: str = ''


@dataclass
class CreateFoldersOutput(TcBaseObj):
    """
    This structure contains information for 'createFolders' operation.
    
    :var clientId: Identifier that helps the client to track the object created.
    :var folder: Folder object reference that was created
    """
    clientId: str = ''
    folder: Folder = None


@dataclass
class CreateFoldersResponse(TcBaseObj):
    """
    Return structure for createFolders operation
    
    :var output: Each element in the list contains a client Id and the related Folder object created.
    :var serviceData: Standard 'ServiceData' member
    """
    output: List[CreateFoldersOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateItemsOutput(TcBaseObj):
    """
    The data structure contains a list of created items and item revisions.
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var item: The created Item object
    :var itemRev: The created ItemRevision object
    """
    clientId: str = ''
    item: Item = None
    itemRev: ItemRevision = None


@dataclass
class CreateItemsResponse(TcBaseObj):
    """
    Return structure for 'createItems' operation.
    
    :var output: A list of the created Item and ItemRevision
    :var serviceData: Service data
    """
    output: List[CreateItemsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateRelationsOutput(TcBaseObj):
    """
    The relations created between the primary and secondary object.
    
    :var clientId: The unmodified value from the 'Relationship.clientId'. This can be used by the caller to indentify
    this data structure with the source input data.
    :var relation: The newly created relation.
    """
    clientId: str = ''
    relation: ImanRelation = None


@dataclass
class CreateRelationsResponse(TcBaseObj):
    """
    'CreateRelationsResponse' structure represents the relations created between the primary and secondary object and
    errors occurred.
    
    :var output: A list of created relations.
    :var serviceData: Standard 'ServiceData' object to hold the partial errors that the operation encounters.
    """
    output: List[CreateRelationsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class DatasetProperties(TcBaseObj):
    """
    Input structure for createDatasets operation
    
    :var clientId: Identifier that helps the client to track the object(s) created, required, should be unique for the
    input set
    :var type: Type of the Dataset to be created
    :var name: Name of the Dataset to be created
    :var description: Description of the Dataset to be created, may be an empty string
    :var toolUsed: Name of the Tool used to open the created Dataset, may be an empty string.
    :var container: Object reference of the container used to hold the created Dataset, may be an empty string
    :var relationType: Name of the relation type for the Dataset to be created, may be an empty string.
    """
    clientId: str = ''
    type: str = ''
    name: str = ''
    description: str = ''
    toolUsed: str = ''
    container: BusinessObject = None
    relationType: str = ''


"""
A map of index to 'ItemIdsAndInitialRevisionIds'.
"""
IndexToIdMap = Dict[int, List[ItemIdsAndInitialRevisionIds]]


"""
IndexToRevIdMap
"""
IndexToRevIdMap = Dict[int, RevisionIds]


"""
ItemRevMap
"""
ItemRevMap = Dict[ItemRevision, ItemRevision]


"""
Map of ItemRevision to 'ReviseProperties' ('ItemRevision', 'ReviseProperties').
"""
ItemRevPropertyMap = Dict[ItemRevision, ReviseProperties]


"""
A map represents the property name and display value pair.
"""
AttributeNameValueMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, POM_object, ValidationResult
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class NameValueStruct(TcBaseObj):
    """
    This structure holds property names and values for each property. The strings for the 'values' element must be
    derived from the Teamcenter::Soa::Client::Property::toXXXString() functions. This will ensure the strings for a
    Date, float, and other property types are formatted correctly.
    
    :var name: The name of the property.
    :var values: The property values.
    """
    name: str = ''
    values: List[str] = ()


@dataclass
class PropInfo(TcBaseObj):
    """
    This structure holds information about object and property information.
    
    :var object: Pointer to an object.
    :var vecNameVal: A vector of structures that contain <property name, property values vector>  pairs of the given
    business object.
    """
    object: BusinessObject = None
    vecNameVal: List[NameValueStruct] = ()


@dataclass
class ReportDatasetInfo(TcBaseObj):
    """
    The DatasetInfo struct represents all of the data necessary to construct the dataset object.
    
    :var clientId: Unique client identifier string.
    :var name: The name for a dataset. If it is an empty string, a name will be automatically generated.
    :var type: The dataset type used to create a dataset. A valid dataset type name string will be required.
    """
    clientId: str = ''
    name: str = ''
    type: str = ''


@dataclass
class ResultOverrideApproval(TcBaseObj):
    """
    The structure contains inputs for performing specified actions for override approval of a validation result.
    
    :var clientId: Unique client identifier
    :var targetBo: A target Business Object whose override approval will be processed.
    :var action: The action to be performed for result override approval.
    1    Create override approval
    2    Update override approval
    3    Approve override approval
    4    Reject override approval
    5    Remove override approval
    :var objectPropNameValues: A vector of PropInfo for result override approval. Empty vector should be given if the
    action is to remove override approval or when the action is to be performed on all applicable ValidationResult
    objects that belong to the given target Business Object.
    """
    clientId: str = ''
    targetBo: POM_object = None
    action: int = 0
    objectPropNameValues: List[PropInfo] = ()


@dataclass
class UpdateInput(TcBaseObj):
    """
    The inputs required for updating an existing ValidationResult object.
    
    :var clientId: Unique client identifier string.
    :var targetBo: The target Business Object whose validation results are to be updated.
    :var removeReportDatasetNamedRefs: The flag indicates whether the named refs of report datasets of ValidationResult
    objects will be removed.
    :var objectPropNameValues: A vector of PropInfo to be used for updating a ValidationResult object.
    """
    clientId: str = ''
    targetBo: POM_object = None
    removeReportDatasetNamedRefs: bool = False
    objectPropNameValues: List[PropInfo] = ()


@dataclass
class ValidationResultInfo2(TcBaseObj):
    """
    The ValidationResultInfo2 structure is the main input to the createOrUpdateValidationResults2 This struct refers to
    the existing ValidationResult, attribute list, report files, and one or more Dataset structs used to create those
    report datasets.
    
    :var createInputVec: A vector of CreateIn  structs required for creating new ValidationResult objects.
    :var updateInputVec: A vector of UpdateInput structs  required for updating existing ValidationResult objects.
    """
    createInputVec: List[CreateIn] = ()
    updateInputVec: List[UpdateInput] = ()


@dataclass
class ValidationResultsOutput2(TcBaseObj):
    """
    The newl created ValidationResult objects are returned.
    
    :var clientId: Unique client identifier string.
    :var resultObject: Pointer to a newly created ValidationResult object.
    """
    clientId: str = ''
    resultObject: ValidationResult = None


@dataclass
class ValidationResultsResponse2(TcBaseObj):
    """
    Holds the response for createOrUpdateValidationResults2.
    
    :var output: The output structure holding the created ValidationResult objects.
    :var serviceData: Holds model objects and partial errors.
    """
    output: List[ValidationResultsOutput2] = ()
    serviceData: ServiceData = None


@dataclass
class CreateIn(TcBaseObj):
    """
    Input for create operation including unique client identifier string.
    
    :var clientId: Unique client identifier string.
    :var targetBo: This object is the validation target Business Object that logically owns the newly created
    ValidationResult object. The concept is that there is an ItemRevision (which is the targetBo for NX Check Mate)
    that owns the ValidationResult that is to be created.
    :var checkerClassName: The checker class name of the referenced validation checker.
    :var paramsCreateInput: The required inputs for creating a ValidationParams object.
    :var reportDataset: Information to create new empty report datasets and attached them to the ValidationResult.
    :var data: Input data for create operation to create a ValidationResult object.
    """
    clientId: str = ''
    targetBo: POM_object = None
    checkerClassName: str = ''
    paramsCreateInput: CreateInput = None
    reportDataset: ReportDatasetInfo = None
    data: CreateInput = None


@dataclass
class CreateInput(TcBaseObj):
    """
    'CreateInput' structure used to capture the inputs required for creation of a validation result business object.
    
    :var boName: Map containing string property values.
    :var stringProps: Map containing string property values.
    :var boolArrayProps: Map containing bool array property values.
    :var dateProps: Map containing DateTime property values.
    :var dateArrayProps: Map containing DateTime array property values.
    :var tagProps: Map containing tag property values.
    :var tagArrayProps: Map containing tag array property values.
    :var compoundCreateInput: CreateInput for compounded objects.
    :var stringArrayProps: Map containing string array property values.
    :var doubleProps: Map containing double property values.
    :var doubleArrayProps: Map containing double array property values.
    :var floatProps: Map containing float property values.
    :var floatArrayProps: Map containing float array property values
    :var intProps: Map containing int property values.
    :var intArrayProps: Map containing int array property values.
    :var boolProps: Map containing bool property values.
    """
    boName: str = ''
    stringProps: StringMap = None
    boolArrayProps: BoolVectorMap = None
    dateProps: DateMap = None
    dateArrayProps: DateVectorMap = None
    tagProps: TagMap = None
    tagArrayProps: TagVectorMap = None
    compoundCreateInput: CreateInputMap = None
    stringArrayProps: StringVectorMap = None
    doubleProps: DoubleMap = None
    doubleArrayProps: DoubleVectorMap = None
    floatProps: FloatMap = None
    floatArrayProps: FloatVectorMap = None
    intProps: IntMap = None
    intArrayProps: IntVectorMap = None
    boolProps: BoolMap = None


"""
CreateInputMap that contains <property name, CreateInput vector> pairs.
"""
CreateInputMap = Dict[str, List[CreateInput]]


"""
DateMap that contains <property name, date value> pairs.
"""
DateMap = Dict[str, datetime]


"""
DateVectorMap that contains <property name, date value vector> pairs.
"""
DateVectorMap = Dict[str, List[datetime]]


"""
DoubleMap that contains <property name, double value> pairs.
"""
DoubleMap = Dict[str, float]


"""
DoubleVectorMap that contains <property name, double value vector> pairs.
"""
DoubleVectorMap = Dict[str, List[float]]


"""
FloatMap that contains <property name, float value> pairs.
"""
FloatMap = Dict[str, float]


"""
FloatVectorMap that contains <property name, float value vector> pairs.
"""
FloatVectorMap = Dict[str, List[float]]


"""
IntMap that contains <property name, integer value> pairs.
"""
IntMap = Dict[str, int]


"""
IntVectorMap that contains <property name, integer value vector> pairs.
"""
IntVectorMap = Dict[str, List[int]]


"""
StringMap that contains <property name, property value> pairs.
"""
StringMap = Dict[str, str]


"""
BoolMap that contains <property name, bool value> pairs.
"""
BoolMap = Dict[str, bool]


"""
StringVectorMap that contains <property name, string value vector> pairs.
"""
StringVectorMap = Dict[str, List[str]]


"""
TagMap that contains <property name, object identifier> pairs.
"""
TagMap = Dict[str, BusinessObject]


"""
TagVectorMap that contains <property name, object identifier vector> pairs.
"""
TagVectorMap = Dict[str, List[BusinessObject]]


"""
BoolVectorMap that contains <property name, bool value vector> pairs.
"""
BoolVectorMap = Dict[str, List[bool]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Vendormanagement._2012_02.VendorManagement import LineItemPropertiesWithType
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class VendorManagementService(TcService):

    @classmethod
    def createOrUpdateLineItemsWithType(cls, properties: List[LineItemPropertiesWithType], bidPackage: BusinessObject) -> ServiceData:
        """
        This service operation creates or updates a group of BidPackageLineItem objects and any subtype of it in the
        context of the mentioned BidPackage. This operation allows the user to find or create a set of
        BidPackageLineItem objects based on the input data. It first tries to find the existence of the specified
        BidPackageLineItem for the specified BidPackage. If the specified BidPackageLineItem is found, then the its
        data will be updated. Otherwise, it will create the specified BidPackageLineItem and associate it to the
        specified BidPackageRevision. If the specified BidPackageLineItem objects exist and any of the input attribute
        values differ from those already set, they will be updated.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateLineItemsWithType',
            library='Vendormanagement',
            service_date='2012_02',
            service_name='VendorManagement',
            params={'properties': properties, 'bidPackage': bidPackage},
            response_cls=ServiceData,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.ProjectManagement._2011_06.ScheduleManagement import ScheduleModifyResponses, LoadProgramViewContainer, ScheduleModifyContainer
from tcsoa.gen.Internal.ProjectManagement._2007_06.ScheduleManagement import LoadProgramViewResponse
from tcsoa.utils import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def loadProgramView(cls, loadProgramViewContainer: LoadProgramViewContainer) -> LoadProgramViewResponse:
        """
        Load a specified program view. Only one program view is accessed at a time. A program view provides a read-only
        view of tasks across multiple schedules. The program view itself consists of multiple schedules and is a
        top-level view of a program consisting of multiple schedules.
        """
        return cls.execute_soa_method(
            method_name='loadProgramView',
            library='Internal-ProjectManagement',
            service_date='2011_06',
            service_name='ScheduleManagement',
            params={'loadProgramViewContainer': loadProgramViewContainer},
            response_cls=LoadProgramViewResponse,
        )

    @classmethod
    def modifySchedules(cls, scheduleModifyContainers: List[ScheduleModifyContainer]) -> ScheduleModifyResponses:
        """
        This operation updates all the affected scheduling objects based on the users request to the Application
        Interface. Schedule properties can be updated. New schedule tasks, dependencies and resource assignments can be
        created through this operation. Existing schedule tasks, dependencies and resource assignments can be updated.
        Existing schedule tasks, dependencies and resource assignments can be deleted.
         The information needed to modify schedule is specified in the 'ScheduleModifyContainer' structure. It returns
        'ScheduleModifyResponses' which contains the response data from the modify request . Errors will be returned in
        the list of partial errors in the 'ServiceData'.
        
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='modifySchedules',
            library='Internal-ProjectManagement',
            service_date='2011_06',
            service_name='ScheduleManagement',
            params={'scheduleModifyContainers': scheduleModifyContainers},
            response_cls=ScheduleModifyResponses,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Awp0XRTObjectSet
from tcsoa.gen.Internal.AWS2._2013_12.DataManagement import GetStyleSheetDatasetInfo, GetStyleSheetClassificationData
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetStyleSheetIn(TcBaseObj):
    """
    Input for the getStyleSheet operation.
    
    :var targetPage: The page in the XRT stylesheet to process.  This field is ignored if the parameter
    processEntireXRT is set to true. This value may be an empty string which would cause the server to process the
    first page in the XRT, or it may be a titleKey for a page in the XRT, for example web_xrt_Overview.  If the target
    page is not found, then the first page in the XRT is processed.
    :var businessObject: The businessObject for which to retrieve a stylesheet.  This field may be NULL.  Typically it
    would only be NULL in the case of CREATE.
    :var businessObjectType: The Teamcenter data type for which to retrieve a stylesheet, if the businessObject
    parameter is NULL, then this field must be populated, otherwise it is ignored.
    :var styleSheetType: The type of stylesheet to return.  Legal values are: SUMMARY, CREATE, RENDERING, or INFO.
    :var styleSheetLastModDate: The last save date of the stylesheet.
    :var clientContext: The map of client context key value entries, where the key is the client and the location in
    the client, and the value is the unique id for the data being presented in that panel.  These values are used to
    process visibleWhen clasuses in the XRT, pages may be enabled or disabled based on where it is being presented in
    the client.
    
    For example valid entries for active workspace may be:
    Key: ActiveWorkspace:Location  
    Value: com.siemens.splm.client.search.SearchLocation
    
    Or
    
    Key: ActiveWorkspace:SubLocation  
    Value: com.siemens.splm.client.search:SavedSearchSubLocation
    """
    targetPage: str = ''
    businessObject: BusinessObject = None
    businessObjectType: str = ''
    styleSheetType: str = ''
    styleSheetLastModDate: datetime = None
    clientContext: StringMap4 = None


@dataclass
class GetStyleSheetOutput(TcBaseObj):
    """
    Output from getStyleSheet operation.
    
    :var processedPage: The name of the page that was processed. If the operation was invoked with processEntireXRT set
    to true, the this field is empty.  If the operation was invoked with processEntireXRT set to false, this this field
    is populated with the page that was processed.  Typically the value of the processedPage would be the same as the
    targetPage, except when the targetPage value is empty or not found in the stylesheet.  In that case, then the first
    page in the stylesheet is processed.
    :var datasetInfo: The information about the stylesheet.
    :var objectSetMap: A Map (string/Teamcenter::Awp0XRTObjectSet) where key is the source field from the stylesheet
    and the value is the data to present inside that objectSet in the application.
    :var localeMap: Map (string/string) where the key is the text, title, or titleKey string from the stylesheet and
    the value is the localized string.
    :var jtFileMap: Map (BusinessObject/string) where the key is the business object to render, and the value is the
    thumbnail ticket for the image to download.
    :var visiblePages: The visible pages in the XRT.  Each bit indicates whether that page can be displayed. Currently
    the maximum number of pages allowed in an XRT is 32.
    :var classificationData: The classification data for the object to be rendered.
    """
    processedPage: str = ''
    datasetInfo: GetStyleSheetDatasetInfo = None
    objectSetMap: ObjectSetMap3 = None
    localeMap: StringMap4 = None
    jtFileMap: ThumbnailMap3 = None
    visiblePages: int = 0
    classificationData: List[GetStyleSheetClassificationData] = ()


@dataclass
class GetStyleSheetResponse(TcBaseObj):
    """
    Response sent to client from the getStyleSheet operation.
    
    :var output: The vector of output information.  One for each input object.
    :var serviceData: The SOA service data.
    """
    output: List[GetStyleSheetOutput] = ()
    serviceData: ServiceData = None


"""
Maps the object set source in the xml rendering style sheet to the business objects.
"""
ObjectSetMap3 = Dict[str, Awp0XRTObjectSet]


"""
String map.
"""
StringMap4 = Dict[str, str]


"""
Maps a business object to a thumbnail file ticket.
"""
ThumbnailMap3 = Dict[BusinessObject, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ItemRevision, BOMLine
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class InsertLevelParameter(TcBaseObj):
    """
    This structure provides a set of input values for the insert level action.
    
    :var bomLines: the selected BOM lines above which to insert a level
    :var itemRevision: the item to insert above the selected BOM lines
    :var viewType: the bom view type
    :var isPrecise: is the BOM view precise?
    """
    bomLines: List[BOMLine] = ()
    itemRevision: ItemRevision = None
    viewType: BusinessObject = None
    isPrecise: bool = False


@dataclass
class MoveNodeParameter(TcBaseObj):
    """
    The structure provides a set of input values for move node action.
    
    :var newParent: the item to attach the pending-cut lines
    :var bomLines: the pending-cut BOM lines to move
    """
    newParent: BOMLine = None
    bomLines: List[BOMLine] = ()


@dataclass
class SplitOccurrenceParameter(TcBaseObj):
    """
    This structure provides a set of input values for the split occurrence action.
    
    :var bomLine: the selected BOM line from which to split occurrence
    :var quantity: the quantity to split from the original BOM lines
    """
    bomLine: BOMLine = None
    quantity: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision, Item, BOMWindow, Effectivity
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetEffectivityGrpListResponse(TcBaseObj):
    """
    Response contains two elements namely 'effGrpRevList' and 'serviceData'. List of Fnd0EffectvtyGrpRevision objects
    applied on the BOMWindow can be retrieved from 'effGrpRevList'. If the BOMWindow does not have any
    Fnd0EffectvtyGrpRevision objects set on it, then the list is empty.
    
    :var serviceData: structure containing error codes and messages
    :var effGrpRevList: Contains list of Fnd0EffectvtyGrpRevision objects applied to the input BOMWindow
    """
    serviceData: ServiceData = None
    effGrpRevList: List[ItemRevision] = ()


@dataclass
class EffectivitiesInputInfo(TcBaseObj):
    """
    Input structure containing details to create, update or delete an Effectivity object for the input
    Fnd0EffectvtyGrpRevision object.
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var effectivityComponent: Effectivity object to be updated or deleted from the Fnd0EffectvtyGrpRevision. If the
    'effectivityComponent' is NULL, new Effectivity object is created for given 'unitRangeText' and 'endItemComponent'.
    :var endItemComponent: Valid Item object representing a product, system or module with respect to which user can
    configure the structure by Effectivity.
    :var unitRangeText: Valid range of unit numbers. Unit range can be discrete or non continuous.
    :var decision: Flag to decide if Effectivity object  is to be updated or deleted. 
    decision = 1 ( Update Effectivity component )
    decision =2 ( Delete Effectivity component )
    There is no flag required for create. If effectivityComponent is given as NULL, new Effectivity object is created.
    """
    clientId: str = ''
    effectivityComponent: Effectivity = None
    endItemComponent: Item = None
    unitRangeText: str = ''
    decision: int = 0


@dataclass
class EffectivityGroupInputInfo(TcBaseObj):
    """
    Input structure containing details of BOMWindow and Fnd0EffectvtyGrpRevision objects to be applied.
    
    :var bomWindow: BOMWindow containing a structure with occurrence effectivity
    :var effGrpRevisions: List of Fnd0EffectvtyGrpRevision objects to be applied to the BOMWindow
    :var clientId: Identifier that helps the client to track the object(s) created
    """
    bomWindow: BOMWindow = None
    effGrpRevisions: List[ItemRevision] = ()
    clientId: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class NamesAndValues(TcBaseObj):
    """
    The NamesAndValues structure represents a generic name-value pair.
    
    :var name: The name of the session option.
    :var value: The value of the session option.
    """
    name: str = ''
    value: str = ''


@dataclass
class OwningSiteAndObjs(TcBaseObj):
    """
    Vector of object and it's owning site.
    These objects need to be remote imported
    
    :var owningSiteId: owningSiteId
    :var objs: objs
    """
    owningSiteId: int = 0
    objs: List[BusinessObject] = ()


@dataclass
class CallToRemoteSiteResponse(TcBaseObj):
    """
    CallToRemoteSiteResponse
    
    :var msgIDs: msgIDs
    :var serviceData: serviceData
    """
    msgIDs: List[str] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Core._2007_12.DataManagement import ListAlternateIdDisplayRulesInfo, ValidateAlternateIdResponse, GetContextAndIdentifiersResponse, ListAlternateIdDisplayRulesResponse, AlternateIdentifiersInput, ValidateAlternateIdInput
from tcsoa.gen.BusinessObjects import BusinessObject, IdDispRule, ImanType
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.Core._2007_12.Session import StateNameValue
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def listAlternateIdDisplayRules(cls, input: ListAlternateIdDisplayRulesInfo) -> ListAlternateIdDisplayRulesResponse:
        """
        Return the current display rule. If the current user flag is set then also return the display rules for the
        current user. If a list of users is supplied, then return the display rules for the list of users; otherwise
        return the display rules for all users.
        
        Use cases:
        Return the current display rule in effect and optionally return all the display rules for the current user.
        Also return the display rules for all users or for a list of users only.
        """
        return cls.execute_soa_method(
            method_name='listAlternateIdDisplayRules',
            library='Core',
            service_date='2007_12',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ListAlternateIdDisplayRulesResponse,
        )

    @classmethod
    def createAlternateIdentifiers(cls, input: List[AlternateIdentifiersInput]) -> ServiceData:
        """
        Create new alternate identifiers. Given an 'IdContext', an 'IdentifierType' and an Item ( and optionally an
        ItemRevision ), create an Identifier object to display an option part number when the 'IdContext' is valid.
        
        Use cases:
        User has a part number for an Item that is used to track the Item. The manufacturer of the Item has a different
        part number. The sales department has another part number. The user needs to keep all 3 part numbers with the
        Item and display them at different times. The user can get a list of 'IdContext' and 'IdentifierTypes' from the
        'getContextsAndIdentifierTypes' operation. Using the 'IdContext' and 'IdentifierType', the client can create an
        Identifer for the Item and ItemRevision to be displayed when the 'IdContext' is valid.
        """
        return cls.execute_soa_method(
            method_name='createAlternateIdentifiers',
            library='Core',
            service_date='2007_12',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def validateAlternateIds(cls, inputs: List[ValidateAlternateIdInput]) -> ValidateAlternateIdResponse:
        """
        Determines if the supplied alternateIds are valid. The USER exit USER_validate_alt_id is used for validation. A
        "modified" alternate id is returned. If the alternate id supplied is valid then the modified one returned is
        the same as the one supplied. If the alternate id supplied is not valid, then the one returned is a valid one.
        
        Use cases:
        The user has an alternate id that they wish to use for an object. The alternate id is sent to this function to
        determine if the new alternate id conforms to the rules defined by the user.
        """
        return cls.execute_soa_method(
            method_name='validateAlternateIds',
            library='Core',
            service_date='2007_12',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=ValidateAlternateIdResponse,
        )

    @classmethod
    def getContextsAndIdentifierTypes(cls, typeTags: List[ImanType]) -> GetContextAndIdentifiersResponse:
        """
        Returns the context and identifier types for the supplied identifiable types.
        
        Use cases:
        A user has defined several IdContexts and Idenitfiers in preparation for creating AlternateIds. This service
        returns the current IdContext and Identifiers allowing the user to select the appropriate data for AlternateId
        creation.
        """
        return cls.execute_soa_method(
            method_name='getContextsAndIdentifierTypes',
            library='Core',
            service_date='2007_12',
            service_name='DataManagement',
            params={'typeTags': typeTags},
            response_cls=GetContextAndIdentifiersResponse,
        )


class SessionService(TcService):

    @classmethod
    def setAndEvaluateIdDisplayRule(cls, identifiableObjects: List[BusinessObject], displayRule: IdDispRule, setRuleAsCurrentInDB: bool) -> ServiceData:
        """
        Set the ID display rule for the session and optionally set it in the database.  The business objects from the
        'identifiableObjects' list are expanded using the new ID display rule and returned. The rule is applied to all
        business objects process throughout the rest of the session.
        """
        return cls.execute_soa_method(
            method_name='setAndEvaluateIdDisplayRule',
            library='Core',
            service_date='2007_12',
            service_name='Session',
            params={'identifiableObjects': identifiableObjects, 'displayRule': displayRule, 'setRuleAsCurrentInDB': setRuleAsCurrentInDB},
            response_cls=ServiceData,
        )

    @classmethod
    def setUserSessionState(cls, pairs: List[StateNameValue]) -> ServiceData:
        """
        Set the desired user session state values.  To clear a field's value, pass an empty string "" as the value.
        Failure to set a particular state value will result in a Partial Error with the clientId set to the name of the
        state property. State values can be per client session or per server session. Client session state is kept
        separate for each client application sharing the same Teamcenter server session, while server session state is
        shared with all client application sharing the Teamcenter server session. Valid keys for the session state
        pairs are:
        
        - currentChangeNotice: The UID of the ChangeNotice business object for this session (client session). This is
        deprecated from release Teamcenter 11.5.
        - refreshPOM: If true the business objects in the POM are refreshed before returning property values.  This
        ensures property data is up-to-date, but is a performance hit (client session).
        - objectPropertyPolicy: The name of the current object property policy. This can also be controlled through the
        ObjectPropertyPolicyManager in the SOA client framework  (client session).
        - maxOperationBracketTime: Time (seconds) to bracket to limit a  POM refresh (client session).
        - maxOperationBracketInactiveTime: Time (seconds) to bracket to limit a  POM refresh (client session).
        - usePolicyOnly: If true, only properties defined in the current Object Property Policy will be returned.
        Objects that are added to the updated list of the ServiceData without named properties by default are returned
        with all properties currently loaded in the POM.
        - formatProperties: If true, the display value of the property will be formatted, if there is an active
        property formatter is attached to it. If false, the display value of the property will not be formatted, even
        if there is an active property formatter attached to it.
        - currentProject: The UID of the Project object (server session).
        - workContex: The UID of the WorkContext object (server session).
        - volume: The UID of the Volume object (server session).
        - local_volume: The UID of the LocalVolume object (server session).
        - groupMember: The UID of the GroupMember object (server session).
        - currentDisplayRule: The UID of the DisplayRule object (server session).
        - currentOrganization: The UID of the Organization object (server session).
        - locationCodePref: The CAGE/Location Code preference. This value is set on the Item attribute
        'fnd0OriginalLocationCode' when Item objects are created (server session).
        - currentChangeItem: The UID of the Change Item Revision business object for this session. This functionality
        is supported from Teamcenter 11.5.
        
        """
        return cls.execute_soa_method(
            method_name='setUserSessionState',
            library='Core',
            service_date='2007_12',
            service_name='Session',
            params={'pairs': pairs},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Group, Role
from typing import Dict, List
from tcsoa.gen.Internal.Administration._2011_06.OrganizationManagement import RoleUsers
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetOrganizationUserMembersResponse(TcBaseObj):
    """
    A list of GroupElement objects containing group membership information in the groups.
    
    :var groupElementMap: Map of Group objects to GroupElement objects.
    :var serviceData: Object with all found GroupMember objects, corresponding Group and Role objects in plain object
    list and possible errors in finding matched GroupMember objects.
    """
    groupElementMap: GroupElementMap = None
    serviceData: ServiceData = None


@dataclass
class GroupElement(TcBaseObj):
    """
    This structure holds the group hierarchical information like the parent of a group and its child groups.
    
    :var group: The Group object of the GroupMember objects.
    :var parent: The parent group of the Group object.
    :var subGroups: The child groups of the Group object which contain matched GroupMember objects.
    :var roles: List of Role objects of the Group object containing matched GroupMember objects.
    :var members: List of GroupMember  objects for a role under the Group object.
    """
    group: Group = None
    parent: Group = None
    subGroups: List[Group] = ()
    roles: List[Role] = ()
    members: List[RoleUsers] = ()


@dataclass
class OrganizationMembersInput(TcBaseObj):
    """
    This structure hold search criteria to find the matching group members.
    
    :var userID: The user_id of User objects matching with GroupMember objects to be found. Teamcenter search  wildcard
     characters are permitted and is case insensitive.
    :var userName: The user_name of User objects matching with GroupMember objects to be found. Teamcenter search 
    wildcard  characters are permitted and is case insensitive
    :var groupName: The  name of Group objects matching with GroupMember objects to be found. Teamcenter search
    wildcard  characters are permitted and is case insensitive.
    :var roleName: The role_name of Role objects matching with GroupMember objects to be found. Teamcenter search
    wildcard  characters are permitted and is case insensitive.
    :var includeInactive: If true, inactive GroupMember objects are included.
    :var includeSubGroups: If true, GroupMember objects from subgroups of matching groups are included.
    """
    userID: str = ''
    userName: str = ''
    groupName: str = ''
    roleName: str = ''
    includeInactive: bool = False
    includeSubGroups: bool = False


"""
This map holds mapping from a Group object to a GroupElement structure.
"""
GroupElementMap = Dict[Group, GroupElement]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AppearancePathInput(TcBaseObj):
    """
    Input parameter to the ComputeAppearancePath service: holds a parent object and a vector of child paths (relative
    to the object) that we would like to compute the AppearancePaths for.
    
    If the childPaths is an empty vector, the service will return all the values for all the children recursively.
    
    
    
    :var parentObject: The parent of the nodes that we would like to get the APN/ absOccUID. If the parent object does
    not have APN, it would be calculated as well. 
    :var childPaths: The paths to the children for which we would like to get the APN\ absOccUID values. The path is
    relative to the parent object.
    If this vector is empty, the return will include the value for all the children recursively. 
    """
    parentObject: BusinessObject = None
    childPaths: List[NodePath] = ()


@dataclass
class AppearancePathResult(TcBaseObj):
    """
    Holds the values of the apnUID and absOccUID properties for the relevant input
    
    :var absOccUID: The value of bl_absocc_uid_in_topline_context property
    :var apnUID: The value of bl_apn_uid_in_topline_context
    """
    absOccUID: str = ''
    apnUID: str = ''


@dataclass
class NodePath(TcBaseObj):
    """
    This struct holds the path to the node to which we would like to comppute the AppearancePath.
    The path is a vector CloneStableID property (bl_occurrence_uid)
    
    :var threadIDs: The path to the node for which we would like to compute the property values. The path is relative
    to the parent (starts from the parentObject).
    The path is given as a vector of strings which represent threadIDs (CloneStableID)
    """
    threadIDs: List[str] = ()


@dataclass
class ComputeAppearancePathResponse(TcBaseObj):
    """
    Results for ComputeAppearancePath service
    
    :var results: Data structure that holds the results
    :var serviceData: Holds all the modified BO and the errors
    """
    results: List[ComputeAppearancePathResult] = ()
    serviceData: ServiceData = None


@dataclass
class ComputeAppearancePathResult(TcBaseObj):
    """
    Holds the input parent object and the matching results for the childPaths 
    The result for childPaths[i] is in childResults[i].
    
    :var parentObject: The object that was given in the input
    :var childResults: The result matching each index in childPaths in the ComputeAppearancePathInput.
    The results holds values for APN and AbcOccID
    """
    parentObject: BusinessObject = None
    childResults: List[AppearancePathResult] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, User
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetGroupRoleViewModelInput(TcBaseObj):
    """
    This structure holds the information about the user object that is used to construct ViewModelObject strucutres and
    information required to fetch correct page of data.
    
    :var userObject: Specifies the User object that will be used to construct list of ViewModelObjects. It can be any
    user if the current logged in user is DBA. For non-dba user it should only be current logged in user.
    :var startIndex: The start index represents the index starting from where the ViewModelObjects should be included
    in the response. And hence this must be a non-negative value. Otherwise all data will be returned and the data will
    not be paginated.
    :var pageSize: The maximum number of ViewModelObjects that should be included while returning the response.
    """
    userObject: User = None
    startIndex: int = 0
    pageSize: int = 0


@dataclass
class GetGroupRoleViewModelResponse(TcBaseObj):
    """
    The structure holds the information about view model objects. Each ViewModelObject structure represents a row
    displayed on the Active Workspace table widget.
    
    :var viewModelRows: A list of view models which can be used to display a row on active     workspace table widget.
    :var totalFound: Total number of nodes that are part of view.
    :var endIndex: Cursor end position for the results returned so far.
    :var serviceData: The service data object.
    """
    viewModelRows: List[ViewModelObject] = ()
    totalFound: int = 0
    endIndex: int = 0
    serviceData: ServiceData = None


@dataclass
class ViewModelObject(TcBaseObj):
    """
    Structure to store the Teamcenter Business object and its associated view model properties to be displayed as a row
    on the Active Workspace table widget.
    
    :var modelObject: Represents the underlying business object of the row.
    :var viewModelProperties: A list of ViewModelProperty structure.
    """
    modelObject: BusinessObject = None
    viewModelProperties: List[ViewModelProperty] = ()


@dataclass
class ViewModelProperty(TcBaseObj):
    """
    Structure to store properties to be displayed as a single cell in a row of Active Workspace table widget. This can
    also include the properties which directly does not belong to the model object.
    
    :var propInternalName: Real name of the property.
    :var propDisplayName: Display name of the property.
    :var propDBValue: Database value of the property.
    :var propUIValue: Value of the property which will be shown to user.
    :var propDataType: Data type of the property.
    :var isEditable: If true, the property is editable; otherwise false.
    :var hasLOV: If true, property has LOV attached; otherwise, false.
    :var isModifiable: If true, the property is modifiable; otherwise, not.
    :var propBO: Business object related to the property. For Example: If property is default_role then it will contain
    Role object. If property is group than it will contain group object.
    """
    propInternalName: str = ''
    propDisplayName: str = ''
    propDBValue: str = ''
    propUIValue: str = ''
    propDataType: str = ''
    isEditable: bool = False
    hasLOV: bool = False
    isModifiable: bool = False
    propBO: BusinessObject = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import VariantRule, Cfg0ConfiguratorPerspective
from tcsoa.gen.Internal.ProductConfiguratorAw._2017_12.ConfiguratorManagement import UserSelectionMap, VariantConfigurationDataResponse, CreateCustomVariantRuleResponse, VariantConfigurationDataInput
from tcsoa.utils import TcService


class ConfiguratorManagementService(TcService):

    @classmethod
    def createCustomVariantRule(cls, configPerspective: Cfg0ConfiguratorPerspective, userSelections: UserSelectionMap, guidedConfigurationMode: bool, variantRule: VariantRule) -> CreateCustomVariantRuleResponse:
        """
        This operation creates adhoc Variant Rule using user selections. An adhoc Variant Rule is purgable by
        purge_adhoc_configuration_contexts utility if referenced by adhoc configuration context (ConfigurationContext)
        
        Exceptions:
        >The following service exception may be returned:
        - 333005 Failed to create Variant Rule. 
        - 333006 configPerspective is null.
        
        """
        return cls.execute_soa_method(
            method_name='createCustomVariantRule',
            library='Internal-ProductConfiguratorAw',
            service_date='2017_12',
            service_name='ConfiguratorManagement',
            params={'configPerspective': configPerspective, 'userSelections': userSelections, 'guidedConfigurationMode': guidedConfigurationMode, 'variantRule': variantRule},
            response_cls=CreateCustomVariantRuleResponse,
        )

    @classmethod
    def getVariantConfigurationData(cls, input: VariantConfigurationDataInput) -> VariantConfigurationDataResponse:
        """
        This operation retrieves variant configuration data required to create a custom variant rule.
        For a given Configurator Context with a list of current variant option selections or a seed variant rule, the
        following is retrieved:
        - The valid selections for the given group by optionally applying constraints
        - All variant option groups available in this Configurator Context
        
        
        
        Exceptions:
        >The following service exception may be returned.
        
        '77073' The operation has failed, because an invalid Configurator Perspective was passed.
        '77074' The operation must contain a product item in the Configurator Perspective.
        '333001' The variant configuration cannot be performed because both the configuration context and the
        configuration perspective must be provided.
        '333002' Configurator context is not found for input selected object.
        """
        return cls.execute_soa_method(
            method_name='getVariantConfigurationData',
            library='Internal-ProductConfiguratorAw',
            service_date='2017_12',
            service_name='ConfiguratorManagement',
            params={'input': input},
            response_cls=VariantConfigurationDataResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from tcsoa.gen.Core._2010_04.DataManagement import BusinessObjectHierarchy
from tcsoa.gen.Core._2014_10.DataManagement import DeepCopyData
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetDeepCopyDataResponse(TcBaseObj):
    """
    Structure that contains the DeepCopyData embedded within which is a recursive data structure. The DeepCopyData
    contains information about how the secondary objects (related and referenced objects) are to be copied (reference,
    copy as object, no copy).
    
    :var selectedBOIsDuplicated: This indicates whether the 'selectedBO' from the method input already has DeepCopyData
    in deepCopyDatas from the method input or not.
    :var deepCopyDatas: A list of DeepCopyData for the 'selectedBO' from the method input. The DeepCopyData object
    contains all the information about how the attached objects are to be copied (Copy as Object, Copy as Reference,
    NoCopy, etc.). DeepCopyData is a recursive data structure that contains the details for the attached objects at the
    next level.
    :var serviceData: Service data containing errors, etc. The plain object list of the Service data is populated with
    the target objects which are to be copied as part of the saveAs/revise operation. If there is an error retrieving
    Business Object for the business object name corresponding to the target object, it is added to the error stack of
    the ServiceData as a partial error.
    """
    selectedBOIsDuplicated: bool = False
    deepCopyDatas: List[DeepCopyData] = ()
    serviceData: ServiceData = None


@dataclass
class GetDomainInput(TcBaseObj):
    """
    Input structure for getDomainOfObjectOrType operation.
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var inputDesignArtifact: Design artifact for which domain information is to be identified and returned. Design
    artifact is of type WorkspaceObject.
    :var typName: Type name  of BusinessObject for which domain information is to be identified and returned. If
    inputDesignArtifact is empty then only  typName will be processed.
    """
    clientId: str = ''
    inputDesignArtifact: WorkspaceObject = None
    typName: str = ''


@dataclass
class LocalizedPropertyValuesInfo2(TcBaseObj):
    """
    This data structure contains business object tag and a list of NameValueLocaleStruct
    
    :var businessObject: The business object
    :var propertyValues: A list of NameValueLocaleStruct that holds property name, value and locale information.
    """
    businessObject: BusinessObject = None
    propertyValues: List[NameValueLocaleStruct2] = ()


@dataclass
class LocalizedPropertyValuesResponse(TcBaseObj):
    """
    This structure contains a list of output localized property value info and partial error.
    
    :var output: A list of structure LocalizedPropertyValuesInfo2 to keep the localized property values info.
    :var partialErrors: Used for storing partial error and standard service data.
    """
    output: List[LocalizedPropertyValuesInfo2] = ()
    partialErrors: ServiceData = None


@dataclass
class NameValueLocaleStruct2(TcBaseObj):
    """
    This data structure contains localization related information for property values.
    
    :var name: Property name (internal)
    :var values: A list of property values
    :var locale: The name of the locale
    :var seqNum: Sequence number
    :var status: A list of localization status values.
    The status must be one of the following values:
    
    For the approved status: "A", "Approved" or the version of the "Approved" string for the client/server log-in
    locale.
    For the in-review status: "R", "In Review", "In-Review", "InReview" or the version of the "In Review" string for
    the client/server log-in locale.
    For the pending status: "P", "Pending" or the version of the "Pending" string for the client/server log-in locale.
    For the invalid status: "I", "Invalid" or the version of the "Invalid" string for the client/server log-in locale.
    For the master status: "M", "Master" or the version of the "Master" string for the client/server log-in locale.
    :var internalStatus: A list of internal values of the status.
    The status must be one of the following values:
    "approved" for the approved status: "A"
    "in review" for the in-review status: "R" 
    "pending" for the pending status: "P" 
    "invalid" for the invalid status: "I" 
    "master" for the master status: "M"
    :var statusDesc: A list of descriptions of statuses used for tooltip on the user interface.
    :var master: Master value indication
    """
    name: str = ''
    values: List[str] = ()
    locale: str = ''
    seqNum: int = 0
    status: List[str] = ()
    internalStatus: List[str] = ()
    statusDesc: List[str] = ()
    master: bool = False


@dataclass
class PropertyNamingruleInfo(TcBaseObj):
    """
    This operation generates values for the given properties of an object(s) during create/revise/save as action based
    on the user exits or naming rules configured on those properties.Customer user exits are given priority over the
    naming rules if both of them are configured on the same property. The counter has to be set active on the naming
    rule in order to generate the next value for a property. This operation also performs naming rule and multi field
    key validation on the generated values and return appropriate partial errors if the validation fails.
    
    For user exit support, an existing user exit will be called to generate values. Right now we support below given
    user exits for corrosponding Objest type.
    
    Object: Item
    User exit name: USER_new_item_id
    Property: item_id
    
    Object: ItemRevision
    User exit name: USER_new_revision_id, USER_new_revision_id_from_alt_rule(Baseline)
    Property: item_revision_id
    
    Object: Dataset
    User exit name: USER_new_dataset_id
    Property: pubr_object_id
    
    Object: Dataset
    User exit name: USER_new_dataset_rev
    Property: rev
    
    Object: Identifier
    User exit name: IDFR_new_alt_id, IDFR_new_rev_id(In Revise case)
    Property: idfr_id
    
    Object: CPD Objects
    User exit name: USER_new_cpd_id
    Property: CPD Objects related property
    
    These each user exits need some specific inputs which are required by them to generate ids. These inputs are part
    of "generateNextValuesIn" structure and are described in details in its description.
    
    
    :var clientId: The unmodified value from the PropertyNamingruleInfo. This can be used by the caller to identify
    this data structure with the source input data. 
    
    
    :var operationType: The type of the operation for which the values are to be generated. 
    1-CREATE
    2-REVISE
    3-SAVEAS.
    
    An error 74007 is returned by the generateNextValues operation if the operationType value is not in any of the
    above. 
    
    :var businessObjectTypeName: The business object type name (for e.g. Item, Dataset or Form etc)   for which unique
    ID generation is required.
    :var propertyNameAttachedPattern: A map of property name and naming rule pattern pairs( string/string ).The key is
    the property name for which the value is to be generated and the value is the naming rule pattern string selected
    for that property. If no pattern is selected for the property the pattern string should be empty string. The caller
    should only pass the properties that are autoassignable.
    :var propertyValuesMap: A map of property name and values list(string/vector) of all the properties that are
    provided as inputs to create/revise/save as action of an object excluding the properties whose values are needed to
    be generated.If a multi field key is configured on the object ,the values of the constituent properties of the
    multi field key other than the ones for which the values are currently being generated are obtained from this map
    in order to perform the validation.
    The entries in this map are also used to generate values for the autoassignable properties of some business objects.
    
    Eg:
    Identifier: "idContext" value is needed to generate "idfr_id" property value
    
    An error 74032 is returned if any of the above property values are not provided .
    
    The values of the properties are to be provided as strings. For non string type properties use the
    Property.toXXXString functions (ie. toDateString) to covert the actual values to a string.
    
    :var additionalInputMap: A map of extra information which is required by user exits to generate values.
    A map where the key (string) is a parameter name and the value (string) is the parameter value.
    
    Valid parameters are:
    
    "ruleSuffix":
    This is only used when Baseline action to be performed. 'PDR' should be passed as ruleSuffix in case of Baseline
    action. An error 74030 is returned if invalid ruleSuffix is supplied.
    
    "sourceObject":
    The object that is being revised/saved or the parent object during Dataset creation . Its value should be passed as
    the object UID. An error 74031 is returned if this parameter is not provided during revise/save as action. It is
    optional in case of Dataset creation .
    
    Object Name        |        Operation                |            Value                    |
    ============================================|
        ITEM                |        Create                    |        NULLTAG                |
                            |        SaveAs                    |        Source tag                |
                            |                                    |                                    |
        ITEM Revision    |        Create/SaveAs            |        NULLTAG                |
                            |        Revise                    |        Source tag                |
                            |        Baseline                    |        Source tag                |
                            |                                    |                                     |
        Identifier            |            Create                |            NULLTAG             |    
                            |        Revise/SaveAs            |         Source tag             |
                            |                                    |                                     |
        Dataset            |         Create                    |         Source tag             |
                            |         SaveAs                |            NULLTAG             |
                            |                                    |                                     |
        CPD Objects    |    Create/Revise/SaveAs    |         Source tag             |
                            |                                    |                                     |
    ============================================|
    
    Any parameter other than the above will not be considered. 
    
    :var compoundObjectInput: Map for compound property.
    A map where the key (string) is a compound property name and the value (vector) is vector of PropertyNamingruleInfo
    type.
    
    By using this map user can generate values for its own properties and also values for any of its compound objects's
    propert. Provided property is autoassignable.
    
    For example user can generate value for "item_id" on item Object and value for "item_revision_id" property also
    using its compound pbject ItemRevision using this map.
    
    :var propertyNameContextListMap: This contains key as property name and value as a list of substituted context list.
    """
    clientId: str = ''
    operationType: int = 0
    businessObjectTypeName: str = ''
    propertyNameAttachedPattern: PropertyAttachedPatternMap = None
    propertyValuesMap: PropertyListOfValuesMap = None
    additionalInputMap: PropertyNamingRuleHelperMap = None
    compoundObjectInput: CompoundPropertyAttachedPatternMap = None
    propertyNameContextListMap: PropertyContextListMap = None


@dataclass
class CreatableBusinessObjectsOut(TcBaseObj):
    """
    This structure contains a list of creatable business object names under a given business object (hierarchy of
    creatable sub business objects).
    
    :var boName: Name of the business object
    :var creatableBONames: Creatable business object hierarchy
    """
    boName: str = ''
    creatableBONames: List[BusinessObjectHierarchy] = ()


@dataclass
class CreatableSubBONamesInput(TcBaseObj):
    """
    Represents the parent business object name, exclusion preference and/or exclusion business object names, and the
    context using which the sub business object names are to be retrieved.
    
    :var boName: The primary business object name for which creatable sub business object names are to be returned. If
    all creatable sub business object names are needed, then "BusinessObject" should be passed as input.
    :var exclusionPreference: Name of the preference to be used to exclude the sub business object names from the
    output. Preference needs to be a multi-valued to specify the names of the business objects to be excluded. This
    parameter is optional.
    :var exclusionBONames: List of business object namess (and their secondary business objects) to be excluded from
    returned list. This parameter is optional.
    :var context: Context based on which server returns the creatable business object names. If there is no value
    specified for the context, all the creatable sub business objects for the business object will be returned.
    Supported contexts:
    legacy: Returns sub business object names from sub classes of the given primary business object, if the primary
    business object name is listed in the TYPE_DISPLAY_RULES_list_types_of_subclasses site preference.
    Please see the Preferences and Environment Variables Reference documentation for more information on preference
    TYPE_DISPLAY_RULES_list_types_of_subclasses.
    """
    boName: str = ''
    exclusionPreference: str = ''
    exclusionBONames: List[str] = ()
    context: str = ''


@dataclass
class CreatableSubBONamesResponse(TcBaseObj):
    """
    This structure holds the list of creatable business objects and their display names
    
    :var output: List of output objects representing the creatable business objects displayable in the Create dialog
    :var serviceData: Service data including the partial errors that are mapped to the input primary business objects
    """
    output: List[CreatableBusinessObjectsOut] = ()
    serviceData: ServiceData = None


@dataclass
class CreateIn2(TcBaseObj):
    """
    This is an input structure for create operation including unique client identifier.
    
    :var clientId: Unique client identifier.
    :var createData: Input data for create operation.
    :var dataToBeRelated: Additional input data. This data (string/list of strings) will be related to the created
    object by the given property.
    :var workflowData: Input data (string/list of strings) required for workflow creation.
    submitToWorkflow - true/false
    NOTE: If the above option is "true" then workflow process template to be used for workflow creation should be
    specified in the preference (<TypeName>_default_workflow_template) defined for the created object type.
    :var targetObject: Target to which the created object will be pasted.
    :var pasteProp: Property to be used to paste the created object to the target.
    """
    clientId: str = ''
    createData: CreateInput2 = None
    dataToBeRelated: PropertyValuesMap2 = None
    workflowData: PropertyValuesMap2 = None
    targetObject: BusinessObject = None
    pasteProp: str = ''


@dataclass
class CreateInput2(TcBaseObj):
    """
    CreateInput2 is a structure used to capture the inputs required for creation of a business object. This is a
    recursive structure containing the CreateInput2 (s) for any secondary (compounded) objects that might be created
    (e.g Item also creates ItemRevision and ItemMasterForm etc.).
    
    :var boName: Business Object type name.
    :var propertyNameValues: Map (string/list of strings)  of property name (key) and to property values (values) in
    string format, to be set on new object being created. Note: The calling client is responsible for converting the
    different property types (int, float, date .etc) to a string using the appropriate function(s) in the client
    framework Property class (i.e. Property.toDateString).
    :var compoundCreateInput: CreateInput2 (s) for compounded objects.
    """
    boName: str = ''
    propertyNameValues: PropertyValuesMap2 = None
    compoundCreateInput: CreateInputMap2 = None


@dataclass
class DeepCopyDataInput(TcBaseObj):
    """
    Input structure for getDeepCopyData operation
    
    :var operation: This is the one of the operation types
    - SaveAs
    - Revise
    
    
    :var targetObject: The target business object which user wants to do save-as/or revise.
    :var deepCopyDatas: This contains the list of DeepCopyData for 'targetObject' which is the object User wants to do
    "saveAs"/"revise".
    :var parentDeepCopyData: This contains parent DeepCopyData of' selctedBO'. It also stores the nested deep copy data
    at the next level.
    :var selectedBO: This is the business object which User wants to change copy action.
    """
    operation: str = ''
    targetObject: BusinessObject = None
    deepCopyDatas: List[DeepCopyData] = ()
    parentDeepCopyData: DeepCopyData = None
    selectedBO: BusinessObject = None


@dataclass
class DomainOfObjectOrTypeResponse(TcBaseObj):
    """
    The identified domain information for the design artifact or type name will be returned.
    
    :var output: The domain information output.
    :var serviceData: Holds any partial errors occurred during the operation.
    """
    output: List[DomainOutput] = ()
    serviceData: ServiceData = None


@dataclass
class DomainOutput(TcBaseObj):
    """
    The identified domain information for input object or type name.
    
    :var clientId: Identifier that helps the client to track the input.
    :var domain: The application domain information identified for the input object or type.
    """
    clientId: str = ''
    domain: str = ''


"""
CreateInputMap is a map of reference or relation property name to secondary CreateInput2 objects.
"""
CreateInputMap2 = Dict[str, List[CreateInput2]]


"""
A map of property name and naming rule pattern pairs( string/string ).The key is the property name for which the value is to be generated and the value is the naming rule pattern string selected for that property. If no pattern is selected for the property the pattern string should be empty string. The caller should only pass the properties that are autoassignable.
"""
PropertyAttachedPatternMap = Dict[str, str]


"""
This is a map contains information required to substitute  the text for a given list of properties. Key is property name and value is substituted text.
The value is a list that needs to be substituted for given pattern for each U, u or D character occurring in naming rule pattern. For example:
Namign Rule Pattern created in  BMIDE is  UuD"-"NNN then valid substitute text value can be "XyB, AbC, ReQ or any Upper Case letter (represented by U in a pattern) followed by lower case letter ( represented by u in a pattern) followed by upper or lower case letter (represented by D in a pattern).

"""
PropertyContextListMap = Dict[str, List[str]]


"""
This map is of property name (as key) and property values (as value) in string format. Each value is a list of strings to support both single valued and multi valued properties of types. The calling client is responsible for converting the different property types (like integer, double, date, etc) to a string using the appropriate to< type >String function (e.g. toIntString and toDateString) in the client framework's Property class. 


"""
PropertyListOfValuesMap = Dict[str, List[str]]


"""
A map of extra information which is required by user exits to generate values.
A map where the key (string) is a parameter name and the value (string) is the parameter value.

Valid parameters are:

"ruleSuffix":
This is only used when Baseline action to be performed. 'PDR' should be passed as ruleSuffix in case of Baseline action. An error 74030 is returned if invalid ruleSuffix is supplied.

"sourceObject":
The object that is being revised/saved or the parent object during Dataset creation . Its value should be passed as the object UID. An error 74031 is returned if this parameter is not provided during revise/save as action. It is optional in case of Dataset creation .

"""
PropertyNamingRuleHelperMap = Dict[str, str]


"""
Map (string/list of strings) of property name to property values in string format, to be set on new object being created. Note: The calling client is responsible for converting the different property types (int, float, date, etc) to a string using the appropriate function(s) in the client framework Property class (i.e. Property.toDateString).
"""
PropertyValuesMap2 = Dict[str, List[str]]


"""
Map for compound property.
A map where the key (string) is a compound property name and the value (vector) is vector of GenerateNextValuesIn type.

By using this map user can generate values for its own properties and also values for any of its compound objects's propert. Provided property is autoassignable.

For example user can generate value for "item_id" on item Object and value for "item_revision_id" property also using its compound pbject ItemRevision using this map.

"""
CompoundPropertyAttachedPatternMap = Dict[str, List[PropertyNamingruleInfo]]



from __future__ import annotations

from tcsoa.gen.Internal.Configurator._2015_10.ConfiguratorManagement import ApplicationConfigExpression
from tcsoa.gen.BusinessObjects import POM_object, Cfg0ConfiguratorPerspective, Cfg0AbsAdmissibility
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetModelOptionCondInput(TcBaseObj):
    """
    Input structure containing the expressions which are to be split in the model and the option variant condition
    components. Each GetModelOptionCondInput forms a partial error boundary.
    
    :var configPerspective: The Cfg0ConfiguratorPerspective instance to specify the context and revision rule. This
    parameter can be NULL if the operation parameter for the perspective is not NULL. If a perspective is specified
    here it overwrites the operation parameter operationConfigPerspective for corresponding processing of
    inputExpressions.
    :var inputExpressions: The List of input expression to be split in the model and the option variant condition
    expressions.
    """
    configPerspective: Cfg0ConfiguratorPerspective = None
    inputExpressions: List[ApplicationConfigExpression] = ()


@dataclass
class GetModelOptionCondOutput(TcBaseObj):
    """
    Output structure containing the model and the option variant condition expressions for the input expressions.
    
    :var index: The index corresponding to input structure index.
    :var modelOptionCondList: The list of structures having condition information one for each input condition.
    """
    index: int = 0
    modelOptionCondList: List[ModelOptionCondStruct] = ()


@dataclass
class GetModelOptionCondResponse(TcBaseObj):
    """
    Response structure for getModelAndOptionConditions operation.
    Each entry will correspond to the partial error boundary of one input structure.
    
    :var modelOptionCondOutputList: The list of output structures one for each input having variant expression
    information.
    :var serviceData: ServiceData to return partial errors.
    """
    modelOptionCondOutputList: List[GetModelOptionCondOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ModelOptionCondStruct(TcBaseObj):
    """
    Structure containing the model expressions and the option variant conditions corresponding to the one input
    ApplicationConfigExpression.
    
    :var modelExpressions: The list of application configuration expression structures for processing.
    :var optionConditions: The list of expression providing information of the option conditions.
    """
    modelExpressions: List[ApplicationConfigExpression] = ()
    optionConditions: List[ApplicationConfigExpression] = ()


@dataclass
class OverlapStateInfo(TcBaseObj):
    """
    Contains information about the degree of overlap between a set of configuration expressions and a given reference
    configuration expression.
    
    :var index: This index identifies the input structure for which overlap states are recorded.
    :var overlapStates: Each overlap state value indicates the degree of overlap between the expression at a given
    index in the corresponding input structure and the reference expression in that input structure.
    The getDisplayStrings operation in the SessionService can be used to obtain a display value for each of the
    following overlap state keys:
    - k_overlapstate_none: The two expressions have no overlap. There is no satisfying solution common to both
    expressions. A conjunction (AND combination) of the two is unsatisfiable.
    - k_overlapstate_subset: The two expressions overlap. The solution set of the expression is a subset of the
    solution set of the reference expression. The conjunction (AND combination) of the expression with the negated
    reference expression is unsatisfiable.
    - k_overlapstate_match: The two expressions are logically equivalent. Every solution that satisfies one expression
    also satisfies the other, and vice versa.
    - k_overlapstate_superset: The two expressions overlap. The solution set of the expression is a superset of the
    solution set of the reference expression. The conjunction (AND combination) of the negated expression with the
    reference expression is unsatisfiable.
    - k_overlapstate_intersect: The two expressions overlap. The solution set of the expression has some overlap with
    the solution set of the reference expression.
    
    """
    index: int = 0
    overlapStates: List[str] = ()


@dataclass
class OverlapStateInput(TcBaseObj):
    """
    Contains a reference expression and a list of configuration expressions with the intent to determine the degree of
    overlap between them.
    
    :var referenceExpression: The reference expression for which the degree of overlap with each of the expressions
    should be determined.
    :var expressions: The list of application configuration expressions for which the overlap state with the
    referenceExpression should be determined.
    """
    referenceExpression: ApplicationConfigExpression = None
    expressions: List[ApplicationConfigExpression] = ()


@dataclass
class OverlapStateResponse(TcBaseObj):
    """
    Returns the degree of overlap between the expressions in each input structure.
    
    :var overlapInfo: Contains overlap state information for the expression in each input record.
    :var serviceData: The standard Teamcenter ServiceData structure.
    """
    overlapInfo: List[OverlapStateInfo] = ()
    serviceData: ServiceData = None


@dataclass
class UpdateAdmissibilityData(TcBaseObj):
    """
    The admissibility data to be updated.
    
    :var admissibilityObject: The admissibility Cfg0AbsAdmissibility object to be updated.
    :var admissibilityState: The new admissibility state to be updated. The admissibility state is driven by LOV
    Cfg0AdmissibilityState.
    """
    admissibilityObject: Cfg0AbsAdmissibility = None
    admissibilityState: str = ''


@dataclass
class UpdateAdmissibilityInputList(TcBaseObj):
    """
    List of update admissibility input.
    
    :var updateAdmissibilityInputs: The list of admissibility input to be updated.
    """
    updateAdmissibilityInputs: List[UpdateAdmissibilityInputStruct] = ()


@dataclass
class UpdateAdmissibilityInputStruct(TcBaseObj):
    """
    The input structure for the operation updateAdmissibility corresponding to one context object.
    
    :var contextObject: The context business object (POM_object) for which admissibility data to be updated.
    :var updateAdmissibilityData: The list of admissibility data corresponding to one context object.
    """
    contextObject: POM_object = None
    updateAdmissibilityData: List[UpdateAdmissibilityData] = ()



from __future__ import annotations

from tcsoa.gen.StructureManagement._2008_05.StructureSearch import OccurrenceNoteExpression, PlaneZone, BoxZoneExpression, AttributeExpression, SearchBySizeExpression, FormAttributeExpression, ProximityExpression, InClassExpression, SavedQueryExpression
from enum import Enum
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class PlaneZoneExpression(TcBaseObj):
    """
    PlaneZoneExpression represents a 3D plane to search against. It may define a search for all parts that intersect
    with the plane, or a search for all parts on one side of the plane.
    
    :var planeZone: It defines a plane to search against
    :var planeZoneOperator: Operator to use for expression
    """
    planeZone: PlaneZone = None
    planeZoneOperator: PlaneZoneOperator = None


@dataclass
class SearchExpressionSet(TcBaseObj):
    """
    SearchExpression data structure specifies the full set of search expressions that are to be used to perform a
    single structure search. Each one of the expressions provided are combined together using a logical 'AND' between
    them.
    
    :var itemAndRevisionAttributeExpressions: A collection of Item and ItemRevision attribute search expressions
    :var occurrenceNoteExpressions: A collection of Occurrence Note attribute search expressions
    :var returnScopedSubTreesHit: Specifies whether the search result should return only the sub-set of
    scopeBomLines&#x0A;given as input as part of the SearchScope
    :var formAttributeExpressions: A collection of Form attribute search expressions
    :var proximitySearchExpressions: A collection of spatial proximity search expressions
    :var boxZoneExpressions: A collection of spatial box zone expressions
    :var planeZoneExpressions: A collection of spatial plane zone search expressions
    :var savedQueryExpressions: A collection of saved query search expressions
    :var inClassQueryExpressions: A collection of inclass attribute search expressions
    :var sizeSearchExpression: A collection of spatial size search expressions
    :var doTrushapeRefinement: Specifies whether to perform trushape (use actual geometry rather using bounding box
    covering min and max points) refinement on the spatial search
    """
    itemAndRevisionAttributeExpressions: List[AttributeExpression] = ()
    occurrenceNoteExpressions: List[OccurrenceNoteExpression] = ()
    returnScopedSubTreesHit: bool = False
    formAttributeExpressions: List[FormAttributeExpression] = ()
    proximitySearchExpressions: List[ProximityExpression] = ()
    boxZoneExpressions: List[BoxZoneExpression] = ()
    planeZoneExpressions: List[PlaneZoneExpression] = ()
    savedQueryExpressions: List[SavedQueryExpression] = ()
    inClassQueryExpressions: List[InClassExpression] = ()
    sizeSearchExpression: SearchBySizeExpression = None
    doTrushapeRefinement: bool = False


class PlaneZoneOperator(Enum):
    """
    PlaneZoneOperator enumerated type defines the operator to use as part of a plane-zone search expression
    """
    PlaneZone_above = 'PlaneZone_above'
    PlaneZone_below = 'PlaneZone_below'
    PlaneZone_intersects = 'PlaneZone_intersects'
    PlaneZone_aboveOrIntersects = 'PlaneZone_aboveOrIntersects'
    PlaneZone_belowOrIntersects = 'PlaneZone_belowOrIntersects'



from __future__ import annotations

from tcsoa.gen.Internal.ActiveWorkspaceBom._2017_06.OccurrenceConfiguration import ConfigRuleResponse, ConfigRuleInput
from tcsoa.gen.Internal.ActiveWorkspaceBom._2017_06.OccurrenceManagement import UserContextState, EffectivityCreateInput, EffectivityEditInput
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class OccurrenceManagementService(TcService):

    @classmethod
    def createAndAddElementEffectivity(cls, input: EffectivityCreateInput) -> ServiceData:
        """
        This operation creates and adds Effectivity to the input Awb0Element. The impacted Awb0Element(s) are
        reconfigured with new effectivity.
        
        Exceptions:
        >This operation may raise a ServiceException containing following   
        errors:
        
        - 126002 No Adapter could be found to handle the request.
        - 126225 The effectivities cannot be updated on Awb0Element.
        - 126228 The effectivity cannot be created.
        
        """
        return cls.execute_soa_method(
            method_name='createAndAddElementEffectivity',
            library='Internal-ActiveWorkspaceBom',
            service_date='2017_06',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def editElementEffectivity(cls, input: EffectivityEditInput) -> ServiceData:
        """
        This operation updates Effectivity of the input Awb0Element. The impacted Awb0Element(s) are reconfigured with
        the updated effectivity
        
        Exceptions:
        >This operation may raise a ServiceException containing following 
        errors:
        - 126002 No Adapter could be found to handle the request.
        - 126225 The effectivity cannot be updated on Awb0Element.
        
        """
        return cls.execute_soa_method(
            method_name='editElementEffectivity',
            library='Internal-ActiveWorkspaceBom',
            service_date='2017_06',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def saveUserWorkingContextState(cls, contextState: UserContextState) -> ServiceData:
        """
        This operation saves current user's client state information for the opened object. The saved information is
        used to establish user's working state while opening the object again.
        """
        return cls.execute_soa_method(
            method_name='saveUserWorkingContextState',
            library='Internal-ActiveWorkspaceBom',
            service_date='2017_06',
            service_name='OccurrenceManagement',
            params={'contextState': contextState},
            response_cls=ServiceData,
        )


class OccurrenceConfigurationService(TcService):

    @classmethod
    def getConfigurationRules(cls, input: ConfigRuleInput) -> ConfigRuleResponse:
        """
        This operation returns a list of RevisionRule or VariantRule based on the input product context information.
        The number of RevisionRule or VariantRule in the response depends on the page size.
        """
        return cls.execute_soa_method(
            method_name='getConfigurationRules',
            library='Internal-ActiveWorkspaceBom',
            service_date='2017_06',
            service_name='OccurrenceConfiguration',
            params={'input': input},
            response_cls=ConfigRuleResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.EntCba._2019_12.Alignments import PartDesOccAlignmentData
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class AlignmentsService(TcService):

    @classmethod
    def alignOccurrences(cls, input: List[PartDesOccAlignmentData]) -> ServiceData:
        """
        This operation aligns the Part and the Design occurrences. If the Part or the Design context is given, the
        alignment is performed using the provided context.  If no context is provided either for the Part occurrence or
        the Design occurrence, alignment is performed with the immediate parent occurrence as the context.
        """
        return cls.execute_soa_method(
            method_name='alignOccurrences',
            library='Internal-EntCba',
            service_date='2019_12',
            service_name='Alignments',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def unalignOccurrences(cls, input: List[PartDesOccAlignmentData]) -> ServiceData:
        """
        This operation unaligns the Part and the Design occurrences. If the Part or the Design context is given, the
        unalignment is performed using the provided context. If no context is provided either for the Part occurrence
        or Design occurrence, unalignment is performed with the immediate parent occurrence as the context.
        """
        return cls.execute_soa_method(
            method_name='unalignOccurrences',
            library='Internal-EntCba',
            service_date='2019_12',
            service_name='Alignments',
            params={'input': input},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GroupName(TcBaseObj):
    """
    The GroupName structure contains the real and display name of the group.
    
    :var realName: Real name of the group.
    :var dispName: Display name of the group.
    :var isModifiable: True if the mapping group can be modified by logged in user.
    """
    realName: str = ''
    dispName: str = ''
    isModifiable: bool = False


@dataclass
class ImportExcelData(TcBaseObj):
    """
    The ImportExcelData structure contains all the data required to import a file to Teamcenter.
    
    :var selectedObject: Selected Object under which the product will be created.
    :var transientFileWriteTicket: Write ticket of the Excel file to be imported to Teamcenter.
    :var mappingGroupData: A list of mapping groups to be consumed during the import.
    :var importOptions: A list of options to be used during the import.
    Supported options are: 
     "ParseHeader" -  The header row in the Excel sheet is    parsed. 
    "RunInBackground" -  The excel file import runs in background mode and user will get a notification when complete.
    If omitted, after completion of import, the top line Revision of the created structure will be opened.
    :var typePropInfos: List of object types and property related information to be retrieved for the types found in
    input Excel sheet. The list is used only when importOption is set to "ParseHeader".
    """
    selectedObject: BusinessObject = None
    transientFileWriteTicket: str = ''
    mappingGroupData: MappingGroupData = None
    importOptions: List[str] = ()
    typePropInfos: List[TypePropInfo] = ()


@dataclass
class ImportExcelResponse(TcBaseObj):
    """
    The ImportExcelResponse structure contains the list of objects of type ItemRevision.
    
    :var revObjects: A list of business objects of type ItemRevision.
    :var serviceData: Service data.
    """
    revObjects: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class MappingGroupData(TcBaseObj):
    """
    The MappingGroupData structure contains the real and display name of the mapping group along with the mapping
    information.
    
    :var groupName: It contains the real and display name of the group.
    :var actionName: Supported action "CREATE", "UPDATE" or "DELETE".
    :var mappingInfo: List of property related information.
    """
    groupName: GroupName = None
    actionName: str = ''
    mappingInfo: List[PropInfo] = ()


@dataclass
class MappingGroupInput(TcBaseObj):
    """
    The MappingGroupInput structure contains all the mapping group names and mapping information.
    
    :var transientFileWriteTicket: The write ticket of the Excel file to be imported to Teamcenter.
    :var mappingGroupData: It contains the action name, group name, and mapping information.
    :var importOptions: List of options used during the import. Supported option: "ParseHeader" - The header row in the
    Excel sheet is parsed.
    """
    transientFileWriteTicket: str = ''
    mappingGroupData: MappingGroupData = None
    importOptions: List[str] = ()


@dataclass
class MappingGroupOutput(TcBaseObj):
    """
    The MappingGroupOutput structure contains the type and property related data that is parsed from the input Excel
    sheet.
    
    :var mappingGroups: List of mapping group related information.
    :var propInfos: List of property related information.
    :var typePropInfos: List of object types and property related information to be retrieved for the types found in
    input Excel sheet. The list is used only when importOption is set to "ParseHeader".
    """
    mappingGroups: List[GroupName] = ()
    propInfos: List[PropInfo] = ()
    typePropInfos: List[TypePropInfo] = ()


@dataclass
class MappingGroupResponse(TcBaseObj):
    """
    The MappingGroupResponse structure contains the list of MappingGroupOutput.
    
    :var mappingOutputs: List of mapping outputs.
    :var serviceData: Service data.
    """
    mappingOutputs: List[MappingGroupOutput] = ()
    serviceData: ServiceData = None


@dataclass
class PropInfo(TcBaseObj):
    """
    The PropInfo structure contains the property related information to be set.
    
    :var propHeader: Name of the property used as column header in Excel sheet.
    :var realPropName: Real name of the property.
    :var dispPropName: Display name of the property.
    :var isRequired: True if it is a required property.
    """
    propHeader: str = ''
    realPropName: str = ''
    dispPropName: str = ''
    isRequired: bool = False


@dataclass
class TypePropInfo(TcBaseObj):
    """
    The TypePropInfo structure is a container to hold the types and property related information.
    
    :var objectType: Valid subclass name of type Item objects to be created.
    :var dispTypeName: Display name of the object type.
    :var propInfos: List of property related information.
    """
    objectType: str = ''
    dispTypeName: str = ''
    propInfos: List[PropInfo] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Awb0Element, Awb0ProductContextInfo
from typing import List
from tcsoa.gen.Internal.ActiveWorkspaceBom._2017_12.OccurrenceManagement import PackedOccurrenceCSIDsResponse
from tcsoa.utils import TcService


class OccurrenceManagementService(TcService):

    @classmethod
    def getPackedOccurrenceCSIDs(cls, productContextInfo: Awb0ProductContextInfo, occurrences: List[Awb0Element]) -> PackedOccurrenceCSIDsResponse:
        """
        Retrieves the clone stable ids of packed occurrences for input occurrences and product configuration
        information.
        
        Use cases:
        User has opened the content in Active Workspace in packed mode. If user selects any occurrence in primary work
        area then viewer should display all packed occurrences of the selected occurrence as well.
        """
        return cls.execute_soa_method(
            method_name='getPackedOccurrenceCSIDs',
            library='Internal-ActiveWorkspaceBom',
            service_date='2017_12',
            service_name='OccurrenceManagement',
            params={'productContextInfo': productContextInfo, 'occurrences': occurrences},
            response_cls=PackedOccurrenceCSIDsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Fnd0AbstractCommand, Fnd0ClientScope, Fnd0Client, Fnd0CommandCollection
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetUIConfigInput(TcBaseObj):
    """
    Contains input information requiredto retrieve UI configurations from the Teamcenter database.
    
    :var clientScopeURIs: List of client scope URIs representing, for example location.sublocation in Active Workspace.
    (Fnd0ClientScope::fnd0ClientScopeURI). If empty, the UI Configuration for all client scopes are returned.
    :var scope: The scope of the desired UI configuration information. This includes the name of the scope (i.e. a
    user, group or role) and scope query parameter information.
    :var client: Client information including client name and hosting client name.
    """
    clientScopeURIs: List[str] = ()
    scope: ScopeInput = None
    client: ClientInput = None


@dataclass
class GetUIConfigResponse(TcBaseObj):
    """
    This structure returns information to the client about column configuration and command applicability. The
    ServiceData contains information about errors encountered during processing.
    
    :var serviceData: ServiceData structure containing errors and command, command collection and icon objects. If
    there is an error retrieving the configuration information, the error added to the ServiceData as a partial error.
    :var uiConfigInfo: List of configuration information including command and column information
    """
    serviceData: ServiceData = None
    uiConfigInfo: List[ClientConfigurations] = ()


@dataclass
class SaveColumnConfigData(TcBaseObj):
    """
    This structure contains column configuration details and a list of column definition information.
    
    :var scope: The scope for which the column data is applicable.  This includes the name of the scope (i.e. a user,
    group or role) and the save scope parameter information.
    :var clientScopeURI: The unique name of the client scope containing configurations.
    :var columnConfigId: The identifier for the column configuration.
    :var sortDirection: Sort order for the columns. Valid values are Ascending and Descending.
    :var columns: Ordered list of column information.
    """
    scope: SaveScopeInput = None
    clientScopeURI: str = ''
    columnConfigId: str = ''
    sortDirection: str = ''
    columns: List[ColumnDefInfo] = ()


@dataclass
class SaveCommandCollectionInfo(TcBaseObj):
    """
    This structure contains information to save for a command collection.
    
    :var collectionId: Unique identifier of the command collection.
    :var configInfo: Configuration changes for the command collection.
    """
    collectionId: str = ''
    configInfo: List[SaveCommandConfigInfo] = ()


@dataclass
class SaveCommandConfigData(TcBaseObj):
    """
    This structure contains information about the command configuration definitions for a client scope URI.
    
    :var scopes: List of scope names for which the command configuration for which the changes are application.  If the
    site scope should be changed scopeName should be empty.
    :var saveCommandCollectionInfo: List of command collections changes.  Each change in this will be applied to each
    scope specified in the scope names list.
    """
    scopes: List[SaveScopeInput] = ()
    saveCommandCollectionInfo: List[SaveCommandCollectionInfo] = ()


@dataclass
class SaveCommandConfigInfo(TcBaseObj):
    """
    Configuration information which indicates if a command or command collection is being hidden or unhidden.
    
    :var isCollection: True if id is the id of a command collection.  False if id is a command.
    :var id: The unique identifier of a command or command collection.
    :var hidden: True if the command or command collection should be hidden.
    """
    isCollection: bool = False
    id: str = ''
    hidden: bool = False


@dataclass
class SaveScopeInput(TcBaseObj):
    """
    SaveScopeInput structure contains information including scope name and details about scope applicability.  The
    value in scopeName should be empty if the value in param refers to Site or LoginUser.
    
    :var scopeName: The name of a valid Teamcenter scope.
    :var saveScopeParam: The save scope that is used to apply some types of UI configurations. Valid values
    are:Site-Save configuration to the Teamcenter site. The scope name should be empty. Group-Save the configuration
    for a Teamcenter Group. Role-Save the configuration for a Teamcenter Role. User-Save the configuration for a
    Teamcenter User. LoginUser-Save the configuration for the current login user. The scope name should be empty.
    """
    scopeName: str = ''
    saveScopeParam: str = ''


@dataclass
class SaveUiConfigurations(TcBaseObj):
    """
    Contains two lists; one specifying column configuration information to save and the other specifying command
    configuraiton information to save.
    
    :var columnConfigurations: List of column configuration information.
    :var commandConfigurations: List of command configuration data.
    """
    columnConfigurations: List[SaveColumnConfigData] = ()
    commandConfigurations: List[SaveCommandConfigData] = ()


@dataclass
class ScopeInput(TcBaseObj):
    """
    Contains scope input information including scope name and scope query param.
    
    :var scopeName: The name of a scope.  For Site and current login user, this value should be empty.
    :var scopeQueryParam: The query scope that is used to retrieve the UI configurations.  Valid values are Site,
    Group, Role, User, LoginUser, and AvailableForLoginUser. Site returns the configuration defined for the site. Group
    returns the configuration defined for a specific Teamcenter Group. Role returns the configuration defined for a
    specific Teamcenter Role. User returns the configurations defined for a specific Teamcenter User. LoginUser returns
    the current configuration defined for the current login user. AvailableForLoginUser returns all the configurations
    available for the current login user.   scopeName should be empty when scopeQueryParam is set for Site, LoginUser
    or AvailableForLoginUser.
    """
    scopeName: str = ''
    scopeQueryParam: str = ''


@dataclass
class ClientConfigurations(TcBaseObj):
    """
    This structure contains command and column configuration data for the indicated client.
    
    :var client: The client for which the configurations are applicable.
    :var columnConfigurations: List of column configurations.
    :var commandConfigurations: List of command configurations.
    """
    client: str = ''
    columnConfigurations: List[ColumnConfigData] = ()
    commandConfigurations: List[CommandConfigData] = ()


@dataclass
class ClientInput(TcBaseObj):
    """
    Specifies client input information including client name and hosting client name.
    
    :var clientName: The name of a client application, as represented by an instance of Fnd0Client in the Teamcenter
    database. This value must match the value of fnd0ClientName property.
    :var hostingClientName: Specifies the name of a hosting client application, as represented by an instance of
    Fnd0Client, in the Teamcenter databases. This value must match a value of the fnd0ClientName property. For example,
    if client A is integrated with client B and the user can invoke client B commands from within client A, the input
    to getUiConfigs would specify client A as hosting client and client B as the client. If the caller wanted native
    commands for client A, client A would be specified as client and hosting client would be empty.
    """
    clientName: str = ''
    hostingClientName: str = ''


@dataclass
class ColumnConfig(TcBaseObj):
    """
    This structure contains information for a column configuration within a client scope URI. It contains a unique
    column config id, a list of column definition information , and default sort direction.
    
    :var columnConfigId: The unique identifier of the column.
    :var sortDirection: How the columns are sorted.  Valid values are Ascending or Descending.
    :var columns: Ordered list of column details.
    """
    columnConfigId: str = ''
    sortDirection: str = ''
    columns: List[ColumnDefInfo] = ()


@dataclass
class ColumnConfigData(TcBaseObj):
    """
    This structure returns information about the column configuration definitions for a scope, hosting client and
    client scope.
    
    :var scope: The scope for which the column data is applicable.
    :var hostingClient: The name of hosting client for which the list of column configurations is applicable. This
    value must correspond to the value of the property fnd0ClientName for an Fnd0Client object in the Teamcenter
    database.
    :var clientScopeURI: The client scope for which the list of column configurations is applicable. This must match a
    value of fnd0ClientScopeURI which is the unique identifier of a client scope (Fnd0ClientScope).
    :var columnConfigurations: List of column configuration details.
    """
    scope: str = ''
    hostingClient: str = ''
    clientScopeURI: str = ''
    columnConfigurations: List[ColumnConfig] = ()


@dataclass
class ColumnDefInfo(TcBaseObj):
    """
    Contains details about a specific column. This includes the type of object for which the column is applicable, the
    name of the property displayed in the column, a flag indicating if the column should be used to order information
    displayed in the client and pixel width.
    
    :var typeName: The Business Object type for the value displayed in the column.   This can be any valid Teamcenter
    business object type.
    :var propertyName: The property name for the value displayed in the column.
    :var sortByFlag: True if the column is used to sort the information displayed to the user.
    :var pixelWidth: The pixel width for the column.  Valid pixel widths are integer values between1 and 500.
    """
    typeName: str = ''
    propertyName: str = ''
    sortByFlag: bool = False
    pixelWidth: int = 0


@dataclass
class CommandCollectionInfo(TcBaseObj):
    """
    Contains a command collection and indexes to its children commands or command collections.
    
    :var childIsCollection: Flag indicating if the child is a command or a command collection.
    :var childConfigIndex: Index of the child in either the CommandConfigData:commands list or the
    CommandConfigData:cmdsCollections list.
    :var commandCollection: The Teamcenter command collection object.
    """
    childIsCollection: List[bool] = ()
    childConfigIndex: List[int] = ()
    commandCollection: Fnd0CommandCollection = None


@dataclass
class CommandConfigData(TcBaseObj):
    """
    This structure returns information about the command configuration definitions for a client scope URI.
    
    :var scopeName: The scope for which the command data is applicable.
    :var hostingClient: The hosting client for which the command data is applicable.
    :var clientScope: The client scope URI containing the list of command configurations.
    :var cmdCollections: List of all top level and child command collections in the client scope.
    :var cmdCollectionIndex: Index into cmdCollections list for top level command collections in the client scope.
    :var commands: List of all command children accessible in the client scope.
    """
    scopeName: str = ''
    hostingClient: Fnd0Client = None
    clientScope: Fnd0ClientScope = None
    cmdCollections: List[CommandCollectionInfo] = ()
    cmdCollectionIndex: List[int] = ()
    commands: List[Fnd0AbstractCommand] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset, ImanRelation, MEAppearancePathNode, GeneralDesignElementLink, GeneralDesignElement, Form, GDEOccurrence, RevisionRule, ItemRevision
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FeatureQueryInfo(TcBaseObj):
    """
    Input to control the QueryPartRelatedFeatures service.
    
    :var clientId: The id string for the operation, used in the return map to specify the return data associated with
    this input set.
    :var itemRevision: The ItemRevision to process
    :var viewType: The  View to be used for PS traversal. If not supplied then the default view will be used.
    :var featureReturnFilter: Information defining the data to be returned.
    """
    clientId: str = ''
    itemRevision: ItemRevision = None
    viewType: str = ''
    featureReturnFilter: FeatureReturnFilter = None


@dataclass
class FeatureReturnFilter(TcBaseObj):
    """
    Filter for how to process.
    
    :var relationTypes: Types of PartToPart relations to process. Required
    :var levelsForward: The number of levels to process PartToPart relations in the primary to secondary direction.
    :var levelsBackward: The number of levels to process PartToPart relations in the secondary to primary direction.
    :var featureTypes: Types of features to be processed, if empty then all.
    :var configurationType: Determines, based on configuration, which item revisions will be processed. 
      All Item Revisions.
      Latest Item Revisions.
      Configured Item Revisions based on configRule.
    :var configRule: Name of a configuration rule to use. REQUIRED when revisionTypes is Configured.
    :var psNotesWanted: A list of names of attributes to retireve from the PSOccurrenceNote attached to the
    GDEOccurrence.
    """
    relationTypes: List[str] = ()
    levelsForward: int = 0
    levelsBackward: int = 0
    featureTypes: List[str] = ()
    configurationType: FeatureConfigurationType = None
    configRule: RevisionRule = None
    psNotesWanted: List[str] = ()


@dataclass
class GDEItem(TcBaseObj):
    """
    GDE Object data
    
    :var gde: The GDE Object.
    :var gdeOccurrence: The GDEOccurrence that references the GDE Object
    :var psOccNotes: The PSOccurrenceNotes for the GDEOccurrence.
    :var apns: The list of APNs associated with the GDE.
    :var featureForm: The FeatureForm associated with the GDE Object, if requested.
    :var numberOfGDELinks: The number of GDE Links that reference the GDE. This may be more than the number of GDELinks
    returned.
    :var gdeLinks: The list of GDELinks that reference the GDE Object,  if requested.
    """
    gde: GeneralDesignElement = None
    gdeOccurrence: GDEOccurrence = None
    psOccNotes: List[GDEOccNote] = ()
    apns: List[MEAppearancePathNode] = ()
    featureForm: Form = None
    numberOfGDELinks: int = 0
    gdeLinks: List[GeneralDesignElementLink] = ()


@dataclass
class GDELinkItem(TcBaseObj):
    """
    Description of a GDELink Object
    
    :var gdeLink: The GDELink Object.
    :var gdeLinkOccurrence: The GDEOccurrence that references the GDELink  Object
    :var psOccNotes: The PSOccurrenceNotes for the GDEOccurrence.
    :var apns: The list of APNs associated with the GDELink.
    :var featureForm: The FeatureForm for the GDELink Object.
    :var numberOfGDEs: The number of GDE referenced by the GDELink. This may be more than the number of GDEs returned.
    :var gdes: The list of GDEs that are referenced by the GDELink Object.
    :var connectedTos: The list of CONNECTED_TO or CF_CONNECTED_TO ImanRelations that the GDELink is in.
    """
    gdeLink: GeneralDesignElementLink = None
    gdeLinkOccurrence: GDEOccurrence = None
    psOccNotes: List[GDEOccNote] = ()
    apns: List[MEAppearancePathNode] = ()
    featureForm: Form = None
    numberOfGDEs: int = 0
    gdes: List[GeneralDesignElement] = ()
    connectedTos: List[ImanRelation] = ()


@dataclass
class GDEOccNote(TcBaseObj):
    """
    A name / value pair for an attribute in an GDEOccurrenceNote.
    
    :var name: The name of the Occurrence Note attribute.
    :var value: The value of the Occurrence Note attribute.
    """
    name: str = ''
    value: str = ''


@dataclass
class PartToPartData(TcBaseObj):
    """
    A PartToPart relation with the primary and secondary objects specified as the comparable item revision.
    
    :var primaryRevision: The equivalent primary ItemRevision in the relationship.
    :var secondaryRevision: The equivalent secondary ItemRevision in the relationship
    :var partToPartRelation: The actual PartToPart IMANRelation.
    """
    primaryRevision: ItemRevision = None
    secondaryRevision: ItemRevision = None
    partToPartRelation: ImanRelation = None


@dataclass
class QueryPartRelatedFeaturesResponse(TcBaseObj):
    """
    Results for the query operations performed.
    
    :var clientIdToFeatureMap: Map of data requested.
    :var serviceData: Service data containing warnings, errors and objects.
    """
    clientIdToFeatureMap: QueryPartRelatedFeaturesMap = None
    serviceData: ServiceData = None


@dataclass
class RevisionData(TcBaseObj):
    """
    The list of item revisions that were processed.
    
    :var itemRevision: The ItemRevision
    :var masterDataset: The UGMaster Datset for the ItemRevision.
    :var gdeItems: The list of GDE Objects owned by the ItemRevision.
    :var gdeLinkItems: The list of GDE Link Objects owned by the ItemRevision
    :var partToPartList: TThe list of PartToPart relations the ItemRevision is part of if the level
    ( i.e. backward or forward count ) has not been reached.
    :var endOfChain: A flag, when true this ItemRevision is at the end of the list as specified by the levels forward
    or backward.
    :var numberOfParents: The number of PartToPart relations where this ItemRevision is the secondary object.
    :var numberOfChildren: The number of PartToPart relations where this ItemRevision is the primary object.
    """
    itemRevision: ItemRevision = None
    masterDataset: Dataset = None
    gdeItems: List[GDEItem] = ()
    gdeLinkItems: List[GDELinkItem] = ()
    partToPartList: List[PartToPartData] = ()
    endOfChain: bool = False
    numberOfParents: int = 0
    numberOfChildren: int = 0


class FeatureConfigurationType(Enum):
    """
    The item revisions to be selected based on configuration.
    """
    All = 'All'
    Configured = 'Configured'
    Latest = 'Latest'


"""
A map of the list of 'RevisionData' from the 'queryPartRelatedFeatures' service. The key is the 'ClientId' supplied in 'FeatureQueryInfo'.
"""
QueryPartRelatedFeaturesMap = Dict[str, List[RevisionData]]



from __future__ import annotations

from tcsoa.gen.Bom._2010_09.StructureManagement import TraversedObjectsInput, ObjectCoverageResponse, ObjectCoverageInput, TraversedObjectsResponse
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def getTraversedObjectsByRule(cls, input: TraversedObjectsInput) -> TraversedObjectsResponse:
        """
        This SOA traverses the structure according to supplied filtering rule and returns the full list of resulting
        lines.
        """
        return cls.execute_soa_method(
            method_name='getTraversedObjectsByRule',
            library='Bom',
            service_date='2010_09',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=TraversedObjectsResponse,
        )

    @classmethod
    def verifyObjectCoverageByRule(cls, input: ObjectCoverageInput) -> ObjectCoverageResponse:
        """
        This SOA verifies whether the received lines fit the supplied filtering rule.
        """
        return cls.execute_soa_method(
            method_name='verifyObjectCoverageByRule',
            library='Bom',
            service_date='2010_09',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=ObjectCoverageResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class AssociateOrRemoveScopeInput(TcBaseObj):
    """
    The structure contains a process line object for which the scopes are added or removed. A list of BOMLine object or
    Mfg0BvrWorkarea  object which are assoiated as scope to the input process line object. A list of already defined
    scope object which is disassociated from the process line object. A generic map to provide additional information
    if required.
    
    :var processLine: A Bill Of Process line (BOPLine) object be associated with the scope. The ItemRevision of this
    BOPLine represents the primary of the relation Fnd0ProcessScopeRel.
    :var addScopeLines: A list of either BOMLine object or Mfg0BvrWorkarea object which are associated with the input
    BOPLine object. The AbsOccurrence of BOMLine  object or the Mfg0BvrWorkarea object represents secondary of
    Fnd0ProcessScopeRel.
    :var removeScopeLines: A list of BOMLine object or Mfg0BvrWorkarea object for which the association with process
    line object is removed. For these objects the Fnd0ProcessScopeRel relation is removed.
    :var additionalInfo: 1. Add scope on multiple process lines. The key is "ScopeProcessLine" and the value is a list
    of process line from BOP structure.
    2. Add scope on child lines of input process line. The key is "ScopeChidLines" and the value is string representing
    "Closure Rule".
    """
    processLine: BusinessObject = None
    addScopeLines: List[BusinessObject] = ()
    removeScopeLines: List[BusinessObject] = ()
    additionalInfo: AdditionalInfo = None


@dataclass
class AdditionalInfo(TcBaseObj):
    """
    A structure containing generic maps to capture the additional information for future use.
    
    :var strToDateMap: A map (string , list of date) to capture additional information.
    :var strToDoubleMap: A map (string, list of double) to capture additional information.
    :var strToStrMap: A map (string, list of string) used to specify if the BOMLine scope is added to the child BOP
    lines of the input process line scope. The key is "ScopeChildLines" and the value is "ClosureRule". The closure
    conatins the BOP object types on which the scope from BOMLine is added.
    :var strToIntMap: A map (string, list of interger) to capture additional information.
    :var strToObjVectorMap: A map (string, list of business object) to capture additional information.
    """
    strToDateMap: StringToDateMap = None
    strToDoubleMap: StringToDoubleMap = None
    strToStrMap: StringToStringMap = None
    strToIntMap: StringToIntegerMap = None
    strToObjVectorMap: StringToObjectMap = None


"""
A map (string , list of date) to capture additional information.
"""
StringToDateMap = Dict[str, List[datetime]]


"""
A map (string, list of double) to capture additional information.
"""
StringToDoubleMap = Dict[str, List[float]]


"""
A map (string, list of interger) to capture additional information.
"""
StringToIntegerMap = Dict[str, List[int]]


"""
A map (string, list of business object) to capture additional information.
"""
StringToObjectMap = Dict[str, List[BusinessObject]]


"""
A map (string, list of string) to capture additional information.
"""
StringToStringMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from tcsoa.gen.Internal.Core._2018_06.LogicalObject import TraversalHop2
from dataclasses import dataclass


@dataclass
class IncludedLogicalObjectDefinition(TcBaseObj):
    """
    An included logical object definition consists of included logical object ID, display name, description and the
    navigation path, which specifies the path to reach the destination logical object.
    
    :var includedLogicalObjectID: ID of the included logical object property
    :var displayName: Display name of the included logical object property.
    :var description: Description of the included logical object property.
    :var traversalPath: A list of traversal hops.
    :var applyConfigurationContext: If TRUE , The destination objects that satisfy the configuration context criteria
    expression will be returned.
    """
    includedLogicalObjectID: str = ''
    displayName: str = ''
    description: str = ''
    traversalPath: List[TraversalHop2] = ()
    applyConfigurationContext: bool = False


@dataclass
class AddIncludedLOInput(TcBaseObj):
    """
    Represents the definition data for included logical objects to be added to a Logical Object Type.
    
    :var logicalObjectType: An object of logical object type ( subtype of "Fnd0LogicalObject" ) to which included
    logical objects are to be added.
    :var includedLODefinitions: A list of included logical object defintions.
    """
    logicalObjectType: BusinessObject = None
    includedLODefinitions: List[IncludedLogicalObjectDefinition] = ()


@dataclass
class AddIncludedLogicalObjectsResponse(TcBaseObj):
    """
    Holds the output list of included logical objects that were created on the specified logical object type and any
    partial errors, if thrown.
    
    :var includedLogicalObjects: List of included logical objects that were created on the specified input logical
    object type.
    :var serviceData: Returned service data.
    """
    includedLogicalObjects: List[BusinessObject] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2016_12.AdvancedSearch import AdvancedQueryCriteriaResponse, AdvancedSearchResponse
from tcsoa.gen.Internal.AWS2._2016_03.Finder import ColumnConfigInput, SearchInput, SaveColumnConfigData
from tcsoa.gen.Internal.AWS2._2016_12.RequirementsManagement import ExportInputDataAsync, ExportToApplicationInputData3, SetContentInput2
from tcsoa.gen.Internal.AWS2._2016_03.RequirementsManagement import ExportToApplicationResponse2
from tcsoa.gen.Internal.AWS2._2016_12.DataManagement import GetDefaultRelationResponse, GetDeclarativeStyleSheetResponse, SaveEditAndSubmitInfo, GetDefaultRelationIn
from tcsoa.gen.Internal.AWS2._2016_04.DataManagement import GetStyleSheetIn
from tcsoa.gen.BusinessObjects import ImanQuery
from typing import List
from tcsoa.gen.Internal.AWS2._2012_10.DataManagement import SaveEditAndSubmitResponse
from tcsoa.gen.Internal.AWS2._2016_12.Finder import SearchResponse2
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class AdvancedSearchService(TcService):

    @classmethod
    def getSelectedQueryCriteria(cls, selectedQuery: ImanQuery) -> AdvancedQueryCriteriaResponse:
        """
        Creates the Advanced Query Criteria business object (Awp0AdvancedQueryCriteria). This object is used to
        populate criteria for currently selected Saved Query. The Query Criteria are represented by dynamic runtime
        business objects. This is necessary because the query criteria fields change deepending on which Saved Query is
        selected by the user. Advanced Query Criteria is the parent business object of the dynamic runtime business
        objects.
        
        Use cases:
        Advanced Search panel is loaded and saved queries are populated in the Advanced Query field. User selects any
        of the listed query, this will retrieve the criteria defined for that query from the database. The criteria
        will be shown on the Advanced Search panel as user input fields so that the user can enter the appropriate
        values to execute the advanced search. This will be called after the getAdvancedSearchInput operation.
        """
        return cls.execute_soa_method(
            method_name='getSelectedQueryCriteria',
            library='Internal-AWS2',
            service_date='2016_12',
            service_name='AdvancedSearch',
            params={'selectedQuery': selectedQuery},
            response_cls=AdvancedQueryCriteriaResponse,
        )

    @classmethod
    def createAdvancedSearchInput(cls) -> AdvancedSearchResponse:
        """
        Creates the Advanced Search Input business object (Awp0AdvancedSearchInput). This object is used to populate
        values for the advanced query name or the quick search name. Based on the quick search name or advanced query
        name that is populated on this object, the quick search criteria or advanced search criteria fields are shown
        in the client UI. The Advanced Search object that is created also has a property with the default quick search
        name populated.
        
        Use cases:
        User clicks on the advanced search link in Active Workspace UI. This brings up a panel that is used to collect
        the initial user input for the quick search name or the advanced search name. This panel is represented by the
        Advanced Search Input business object and its properties that is instantiated by this operation.
        """
        return cls.execute_soa_method(
            method_name='createAdvancedSearchInput',
            library='Internal-AWS2',
            service_date='2016_12',
            service_name='AdvancedSearch',
            params={},
            response_cls=AdvancedSearchResponse,
        )


class FinderService(TcService):

    @classmethod
    def performSearch2(cls, searchInput: SearchInput, columnConfigInput: ColumnConfigInput, saveColumnConfigData: SaveColumnConfigData) -> SearchResponse2:
        """
        This operation routes a search request to a specific provider specified as providerName in the searchInput
        structures
        
        The framework allows a custom solution to be able to provide a new specific provider to collect data, perform
        sorting and filtering. The provider can be a User Inbox retriever or Full Text searcher, for example. The new
        data provider can be encapsulated via a new runtime business object from Fnd0BaseProvider class. The
        implementation is done using its fnd0performSearch operation.
        
        RuntimeBusinessObject
        ---- Fnd0BaseProvider (foundation template)
        -------- Fnd0GetChildrenProvider(foundation template)
        -------- Awp0FullTextSearchProvider (aws template)
        -------- Awp0TaskSearchProvider (aws template)
        -------- Aos0TestProvider (aosinternal template)
        -------- etc.
        This operation provides a common interface to send the request to and receive the response from a new data
        provider. Ultimately it allows common framework in UI to support filter, pagination, and sorting.
        
        This operation allows the caller to send the search input, filter values, and sorting data. These input values
        are then to be passed to the fnd0performSearch operation on the server to collect, sort, and filter its results.
        
        The first two input parameters are important. The first input is the provider name. This is a string to
        represent the type name of RuntimeBusinessObject which this request should be routed to. If the template that
        contains the class is not installed, a partial error 217016 is returned. The second input is the search input
        map. The key is different per each provider. For example, for Full Text searcher, the input key would be a
        searchString. For User Inbox retriever, the input key would be an Inbox Type. The fnd0performSearch
        implementation for each provider takes into account the key name as it is used to store the values in
        OperationInput object. 
        
        The internal property name is passed as part of search input to allow grouping of business objects. The
        grouping allows for added functionality such as color coding, etc. If no internal property name is passed,
        default value is assumed as "object_type" to group the business objects.
        
        If new list of column config data is passed in to be saved, it will get saved in the database for the login
        user scope. This will be done prior to performing the search. 
        
        The column configuration input has the criteria needed to query for the columns configured for the provided
        client and client scope URI. The final list of columns are identified based on the types returned back in the
        search results and the operation type( Union / Intersection / Configured ) configured for the client scope URI.
        Finally the determined column attributes and the attributes to be inflated passed in as part of search input
        criteria are combined. This combined list of attributes along with the business objects found during search are
        added to the Service data.
        
        Use cases:
        Use Case 1: User clicks on Inbox tile
        A search request will be made by passing in the needed criteria. In this case the the provider name is
        "Awp0InboxProvider". It queries for the data and finds the effective columns for the client scope URI passed in
        through column config input. The results are finally shown in the view type selected in client. If table view
        is selected the returned effective columns will be shown.
        
        Use Case 2: User performs search by entering a keyword in global search input box.
        A search request will be made by passing in the needed criteria that includes the keyword user has typed in. In
        this case the the provider name is "Awp0FullTextSearchProvider". It queries for the data and finds the
        effective columns for the client scope URI passed in through column config input. The results are finally shown
        in the view type selected in client. If table view is selected the returned effective columns will be shown.
        
        Use Case 3: User tries to save new column configuartions after performing search.
        The new list of column configurations will be saved for the login user scope. Later the search request will be
        made by passing in the needed criteria that includes the keyword user had typed in. In this case the the
        provider name is "Awp0FullTextSearchProvider". It queries for the data and finds the effective columns for the
        client scope URI passed in through column config input. The results are finally shown in the view type selected
        in client. If table view is selected the returned effective columns will be shown.
        """
        return cls.execute_soa_method(
            method_name='performSearch2',
            library='Internal-AWS2',
            service_date='2016_12',
            service_name='Finder',
            params={'searchInput': searchInput, 'columnConfigInput': columnConfigInput, 'saveColumnConfigData': saveColumnConfigData},
            response_cls=SearchResponse2,
        )


class DataManagementService(TcService):

    @classmethod
    def saveEditAndSubmitToWorkflow(cls, inputs: List[SaveEditAndSubmitInfo]) -> SaveEditAndSubmitResponse:
        """
        This operation saves the modified properties for the given input objects and submits the objects to a workflow.
        The workflow is submitted only if all of the save operations are successful. If the save fails for a single
        object  none of  the input objects will be submitted to a workflow.
        
        Use cases:
        User can modify the object(s) properties and submit the object(s) to workflow in one operation. This operation
        first saves the modified properties and then initiates the workflow process for all input objects.
        """
        return cls.execute_soa_method(
            method_name='saveEditAndSubmitToWorkflow',
            library='Internal-AWS2',
            service_date='2016_12',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=SaveEditAndSubmitResponse,
        )

    @classmethod
    def getDeclarativeStyleSheets(cls, processEntireXRT: bool, input: List[GetStyleSheetIn]) -> GetDeclarativeStyleSheetResponse:
        """
        This operation returns the declarative UI definition generated from stylesheets for each input object. It first
        gets the stylesheets for each input object, then converts them into declarative view defintions which can be
        rended as HTML page by declarative UI framework.
        
        Use cases:
        Use Case 1: Load Create Object Panel
        When creating an object of selected type in creation panel, the getDeclarativeStyleSheets SOA operation is
        invoked, and the returned data is parsed/processed in the declarative UI framework to display the creation
        panel.
        
        Use Case 2: Show Object Info
        When an object is selected in ActiveWorkspace, a user may choose the "Show Object Info" command.  When
        executed, a panel slides out from the right hand side of the application, and the selected objects data is
        presented.  In order to populate the panel, the getDeclarativeStyleSheets call is invoked and the returned data
        is parsed and processed in order to build the declarative UI.
        """
        return cls.execute_soa_method(
            method_name='getDeclarativeStyleSheets',
            library='Internal-AWS2',
            service_date='2016_12',
            service_name='DataManagement',
            params={'processEntireXRT': processEntireXRT, 'input': input},
            response_cls=GetDeclarativeStyleSheetResponse,
        )

    @classmethod
    def getDefaultRelation(cls, input: List[GetDefaultRelationIn]) -> GetDefaultRelationResponse:
        """
        This operation returns the value of default relation preference for given combination of primary and secondary
        object types as defined by Teamcenter Administrator. The entire type hierarchy of secondary object type is
        traversed while determining the default relation preference. The preference format should be as follows:
        <primary_object_type>_<secondary_object_type>_default_relation
        
        If the secondary object type is passed as empty, then the preference format should be as follows:
        <primaty_object_type>_default_relation
        
        Use cases:
        Use Case 1: Preference for Primary and Secondary object type default relation exists
        Teamcenter Administrator defines the default relation betweeen ItemRevision and DocumentRevision to be
        IMAN_manifestation through the following preference:
        
        ItemRevision_DocumentRevision_default_relation : IMAN_ manifestation
        
        When getDefaultRelation operation of DataManagement service is invoked passing primary object type as
        "ItemRevision" and secondary object type as "DocumentRevision", the response returned is "IMAN_manifestation".
        
        Use Case 2: Preference for Primary and Parent of Secondary object type default relation exists
        Teamcenter Administrator defines the default relation between ItemRevision and DocumentRevision to be
        IMAN_manifestation through the following preference:
        
        ItemRevision_DocumentRevision_default_relation : IMAN_ manifestation
        
        When getDefaultRelation operation of DataManagement service is invoked passing primary object type as
        "ItemRevision" and secondary object type as "EmailRevision", the operation will check preference value for
        given combination of types. In case the preference is not defined for given pair, the operation will try
        finding preference value for combination of  "ItemRevision" and each type in "EmailRevision" business object's
        type hierarchy. EmailRevision inherits DocumentRevision and since the preference for primary type as
        "ItemRevision" and secondary type as "DocumentRevision" is defined, the  default relation in response is
        returned as "IMAN_manifestation". 
        
        Use Case 3: Preference for Primary object type default relation exists
        Teamcenter Administrator defines the default relation for ItemRevision to be IMAN_specification through the
        following preference:
        
        ItemRevision_default_relation : IMAN_ specification
        
        When getDefaultRelation operation of DataManagement service is invoked passing primary object type as
        "ItemRevision" and secondary object type as "DocumentRevision", the operatin will check for preference value
        for input types. In case the preference value for given combination and any combination from secondary type
        hierarchy is not found, the operation will return preference value for "ItemRevision_default_relation" as
        "IMAN_specification".
        """
        return cls.execute_soa_method(
            method_name='getDefaultRelation',
            library='Internal-AWS2',
            service_date='2016_12',
            service_name='DataManagement',
            params={'input': input},
            response_cls=GetDefaultRelationResponse,
        )


class RequirementsManagementService(TcService):

    @classmethod
    def setRichContent2(cls, inputs: List[SetContentInput2]) -> ServiceData:
        """
        This operation sets the contents of the FullText or SpecElementRevision object. The input content type can be
        'HTML', 'Plain' or 'RichText' type. All modified objects and exceptions are added to the returned service data.
        Additionally this operation will prevent the user from overwriting the contents of FullText or
        SpecElementRevision object, when multiple users are trying to update contents of same object by comparing last
        saved date of FullText object.
        
        Use cases:
        1. User selects a object of type SpecElement created from Active Workspace client and clicks on the Viewer tab.
        The HTML contents of the selected SpecElement type is shown in the Viewer tab. User can set rich text contents
        of Requirement object by using setRichContent2 operation.
        
        2. When multiple users edit a requirement's content concurrently and try to save using setRichContent2
        operation. It prevent user from overwriting changes of another user for the same requirement by comparing last
        saved date of FullText object.
        """
        return cls.execute_soa_method(
            method_name='setRichContent2',
            library='Internal-AWS2',
            service_date='2016_12',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def exportAsync(cls, inputs: ExportInputDataAsync) -> None:
        """
        This operation exports the selected occurrences in the structure asynchronously to an application such as
        MSExcel. The behavior of export operation is controlled by applicationFormat argument specified in
        ExportInputDataAsync structure. The objects may be checked out during the export.
        
        Following are supported application formats from input.
        
        - MSExcelLive - This mode provides the capability of editing of properties of  underlying ItemRevision in
        MSExcel and save the changes to Teamcenter. The generated sheet allows user to login and change the properties
        from MSExcel application. The changes will be saved to Teamcenter when user exits out of the cell in MSExcel
        application.  Teamcenter extensions for Microsoft Office should be installed on the client machine to support
        this application format.
        - MSExcel - This mode provides the capability of exporting objects to static MSExcel application. 
        - MSExcelReimport - This mode provides the capability of exporting objects to MSExcel application for reimport
        purpose. The generated sheet can be used to add/remove structure elements or to update the properties on the
        SpecElementRevision. The generated sheet should be reimported back to Teamcenter. Office Client should be
        installed on the client machine to support this application format
        
        
        
        Use cases:
        - User loads a structure in ACE (Active Content Experience) panel. The ACE panel is used for smart navigation
        of occurrences in a structure.User selects an occurrence and clicks on the Export to Excel button in the
        toolbar. This will show a panel with options to "View", "Live Edit" and "Edit and Import". This panel allows
        user to select a template and if user clicks on Export button, an Excel sheet will be generated. If user has
        selected a single occurrence and clicks on export, then the occurrence and all its children are exported to
        MSExcel. If user has selected multiple occurrences and clicks on export, then the selected occurrences are
        exported to MSExcel.
        - User selects single or multiple objects in Active Workspace Client. User clicks on the Export to Excel button
        in the toolbar. User can select View, Edit or Reimport option. Depending upon the option selected, a static
        MSExcel report or a Live MSExcel sheet or a sheet is generated that can be reimported back to Teamcenter.
        - User selects an ItemRevision and clicks on the Export to Excel button on the toolbar. If user selects "Check
        out" option then selected ItemRevision and its children will be checked out and exported to MSExcel.
        - User selects single or multiple objects in ActiveWorkspace Client. User selects the background checkbox in
        the export panel and clicks on the Export to Excel button in the toolbar. This will invoke the Export to Excel
        using the async service options. When the Export to Excel completes, a notification is sent to the client to
        download the Excel file. The generated file is also attached to the MSExcelX dataset.
        - User selects the "apply_bom_crawling" LOV value on the ExcelTemplate and selects an occurrence in ACE and
        clicks on Export to Excel button in the toolbar. This will crawlback from the selected object to its parent's
        recursively till its top line and the exported structure will export the selected line along with its parents.
        
        """
        return cls.execute_soa_method(
            method_name='exportAsync',
            library='Internal-AWS2',
            service_date='2016_12',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=None,
        )

    @classmethod
    def exportToApplication2(cls, input: List[ExportToApplicationInputData3]) -> ExportToApplicationResponse2:
        """
        This operation exports  the selected occurrences in the structure to an application such as MSWord and MSExcel.
        The behavior of export operation is controlled by applicationFormat argument specified in
        ExportToApplicationInput structure. Additionlly this operation allows callers  of the SOA  to supply a
        parameter to let the objects be checked out during export.will allow Teamcenter objects to be checked out.  The
        objects may be checked out during the export. 
        
        Following are supported application formats from input.
        
        -  MSExcelLive - This mode provides the capability of editing of properties of  underlying ItemRevision  in
        MSExcel and save the changes to Teamcenter. The generated sheet allows user to login and change the properties
        from MSExcel application. The changes will be saved to Teamcenter when user exits out of the cell in MSExcel
        application.  Teamcenter extensions for Microsoft Office should be installed on the client machine to support
        this application format.
        - MSWordXMLLive - This mode provides the capability of structural editing of SpecElementRevisions in MSWord.
        Users can edit the rich text of Requirement and perform structural changes to the exported Requirement(s) in
        the word document. Office Client should be installed on the client machine to support this application format.
        -  MSExcel - This mode provides the capability of exporting objects to static MSExcel application. 
        -  MSExcelReimport- This mode provides the capability of exporting objects to MSExcel application for reimport
        purpose. The generated sheet can be used to for add/remove structure elements or to update the properties on
        the SpecElement revision. The generated sheet should be reimported back to  Teamcenter. Office Client should be
        installed on the client machine to support this application format.
        -  MSWordCompare - This mode provides the capability of comparing the contents of source and target objects
        using MSWord application.  The comparison document is generated asynchonously in the background and user is
        notified when the comparison document is generated. Dispatcher and notification services should be running to
        support this application format.
        -  HTML- This mode provides the capability of fetching the HTML contents of input SpecElement objects.
        
        
        
        This operation is for Teamcenter internal use and is unpublished.
        
        Use cases:
        1)  User loads a structure in ACE (Active Content Experience) panel. The ACE panel is used for smart navigation
        of occurrences in a structure.
        User selects an occurrence and clicks on the Export To Word or Export to Excel button in the toolbar. This
        action will allow users to select a template that will be used for export purpose. If user has selected a
        single occurrence and clicks on export, then the occurrence and all its children are exported to MSWord or
        MSExcel. If user has selected multiple occurrences and clicks on export, then the selected occurrences are
        exported to MSWord or MSExcel.
        
        2)  User selects single or multiple objects in Active Workspace Client. User clicks on the Export to Excel
        button in the toolbar. User can select View, Edit or Reimport option. Depending upon the option selected, a
        static MSExcel report or a Live MSExcel sheet or a sheet is generated that can be reimported back to
        Teamcenter. 
        3)  User selects single or multiple objects in Active Workspace Client and sets them as source objects to
        compare. Similarly user selects single or multiple target objects to compare. User clicks on the Export to Word
        button in the toolbar along with the checkbox for compare contents. User selects the compare contents checkbox
        that allows user to compare the MSWord contents of source and target objects. User is notified when the
        comparison document is generated and attached to a dataset.
        4)  User selects a SpecElement created from Active Workspace client and clicks on the Viewer tab. The HTML
        contents of the selected SpecElement is shown in the Viewer tab.
        5)  User selects an   ItemRevision  and clicks on the Export To Word or Export to Excel button in on the
        toolbar. If user selects "Check out" option then selected  occurrences ItemRevision and its children will be
        checked out and exported to MSWord or MSExcel.
        """
        return cls.execute_soa_method(
            method_name='exportToApplication2',
            library='Internal-AWS2',
            service_date='2016_12',
            service_name='RequirementsManagement',
            params={'input': input},
            response_cls=ExportToApplicationResponse2,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, VisStructureContext, BOMLine, BOMWindow
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExpandPSData(TcBaseObj):
    """
    Through this structure, the child BOMLine, the object of the BOMLine and the objects attached to the BOMLine via
    specified relations are returned.
    
    :var bomLine: BOMLine representing a particular occurrence.
    :var objectOfBOMLine: The object that the BOMLine represents.
    :var relatedObjects: List of object references attached to the BOMLine with the given relations.
    """
    bomLine: BOMLine = None
    objectOfBOMLine: BusinessObject = None
    relatedObjects: List[ExpandPSRelatedObjectInfo] = ()


@dataclass
class ExpandPSFromOccurrenceListInfo(TcBaseObj):
    """
    Input structure that defines the occurrences which are to be expanded into the BOM window
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    
    :var occurrences: List of occurrences that are to be expanded.
    """
    clientId: str = ''
    occurrences: List[OccurrenceListInfo] = ()


@dataclass
class ExpandPSFromOccurrenceListOutput(TcBaseObj):
    """
    This structure contains the client identifier and the list of 'OccurrenceListResults'.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    
    :var occurrenceList: List of 'OccurrenceListResults' structs describing the occurrences that were expanded
    """
    clientId: str = ''
    occurrenceList: List[OccurrenceListResults] = ()


@dataclass
class ExpandPSFromOccurrenceListPref(TcBaseObj):
    """
    A structure that allows for filtering criteria to be specified.
    
    :var wantDatasets: Boolean flag that when true causes the item revision to expand further so that its datasets can
    be found. If the info argument is empty, all the datasets related to the 'objectOfBOMLine' are returned. Otherwise,
    the filtering rules will be honored. If false, no related objects will be returned.
    :var info: Contains the list of relation name and the types of objects of the given relation to return along with
    the children. If no 'RelationAndTypesFilter' is supplied and the 'wantDatasets' is true, then all Datasets related
    to the 'objectOfBOMLines' are returned.
    """
    wantDatasets: bool = False
    info: List[RelationAndTypesFilter] = ()


@dataclass
class ExpandPSFromOccurrenceListResponse(TcBaseObj):
    """
    Response structure for the 'expandPSFromOccurrenceList' operation.
    
    :var output: List of 'ExpandPSFromOccurrenceListOutput' structures containing the objects corresponding to the
    parent and the child BOMLines and their related objects as well.
    :var serviceData: The service data contains the plain objects and the error stack, if any. Error 46061 is returned
    when the related object is unreadable because of access rules and the user does not have read access to the object.
     Error 46063 is returned when the related object is a remote item and not available at the local site.
    """
    output: List[ExpandPSFromOccurrenceListOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ExpandPSNamedReferenceInfo(TcBaseObj):
    """
    This structure is used to identify the reference object corresponding to the named reference.
    
    :var namedReferenceType: The type of the reference object.
    :var namedReferenceName: The name of the reference object.
    :var referenceObject: Object reference corresponding to the named reference.
    :var fileTicket: FMS ticket used to retrieve the file in cases where referenceObject is a file.
    """
    namedReferenceType: str = ''
    namedReferenceName: str = ''
    referenceObject: BusinessObject = None
    fileTicket: str = ''


@dataclass
class ExpandPSParentData(TcBaseObj):
    """
    Through this structure, the parent BOMLine , the object of the bom line and the objects attached to the bom line
    object are returned.
    
    :var bomLine: BOMLine of the parent to be expanded.
    :var objectOfBOMLine: The object that the parent BOMLine represents.
    :var parentRelatedObjects: List of object references attached to the parent with the given relations.
    """
    bomLine: BOMLine = None
    objectOfBOMLine: BusinessObject = None
    parentRelatedObjects: List[ExpandPSRelatedObjectInfo] = ()


@dataclass
class ExpandPSRelatedObjectInfo(TcBaseObj):
    """
    This structure associates a related object, named references and reference objects.
    
    :var relatedObject: This object is the related to the BOMLine by one of the relation specified in the input
    reference.
    :var namedRefList: List of named references and reference objects of the 'relatedObject'.
    """
    relatedObject: BusinessObject = None
    namedRefList: List[ExpandPSNamedReferenceInfo] = ()


@dataclass
class AreRecipesMergableInfo(TcBaseObj):
    """
    Input structure that contains a recipe object (with optional topline) and the
    BOM window to compare against.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements 
     and partial errors associated with this input structure.
    
    :var recipeObj: Object of type ConfigurationContext or StructureContext that records the configuration recipe.
    
    :var topline: Top line object that must be non null if recipeObj is of type ConfigurationContext. This object is
    either the BOMView or BOMViewRevision of the top line.  This argument is not used when the recipeObj is of type
    StructureContext because StructureContext objects contain a reference to their own top line object.
    
    :var bomWindow: The BOMWindow reference.
    """
    clientId: str = ''
    recipeObj: BusinessObject = None
    topline: BusinessObject = None
    bomWindow: BOMWindow = None


@dataclass
class AreRecipesMergableOutput(TcBaseObj):
    """
    The output structure that contains the client identifier and a boolean
    indicating whether or not a merge is allowed.
    
    :var clientId: The unique string supplied by the caller used to match the output to the supplied input.
    :var isMergable: The answer to the question of whether the recipe contained in the configuration object is
    equivalent to the given BOMWindow configuration.
    """
    clientId: str = ''
    isMergable: bool = False


@dataclass
class AreRecipesMergableResponse(TcBaseObj):
    """
    Response structure for the 'areRecipesMergable' operation.
    
    :var output: List of 'AreRecipesMergableOutput' structures containing the Boolean answer as  to whether the recipe
    contained in the configuration object is equivalent to the BOMWIndow configuration.  If true then the ocurrences in
    the configuration object may be merged into the given BOMWindow.
    
    :var serviceData: The Service Data.
    """
    output: List[AreRecipesMergableOutput] = ()
    serviceData: ServiceData = None


@dataclass
class OccurrenceChain(TcBaseObj):
    """
    Input structure that defines a single occurrence uid chain from the topline (but not including topline) to the
    BOMLine that this chains represents.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    
    :var occurrenceChainStr: List of occurrence UIDs that make up the chain of UIDs from the parent BOMLine (but not
    including the parent) down to the last occurrence in the chain.
    """
    clientId: str = ''
    occurrenceChainStr: List[str] = ()


@dataclass
class OccurrenceChainList(TcBaseObj):
    """
    Input structure that defines the parent context and a list of occurrence
    chains to expand within that context.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    
    :var parentBomLine: BOMLine reference that is to be used as the parent line when the occurrences in the
    'occurrenceList' are expanded.
    
    :var occurrenceList: The list of occurrences to be expanded and the associated BOMLines returned. These occurrences
    are represented as a list of UID strings. The BOMLine property used to originally obtain these UIDs are listed in
    the 'attibuteNames' member of the 'OccurrenceListInfo' structure.
    """
    clientId: str = ''
    parentBomLine: BOMLine = None
    occurrenceList: List[OccurrenceChain] = ()


@dataclass
class OccurrenceChainResult(TcBaseObj):
    """
    This structure contains the client identifier and the list of 'ExpandPSData' structures for each occurrence in a
    given chain.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    
    :var occurrenceChain: The list of 'ExpandPSData' structures that describe the occurrences that were expanded in the
    given occurrence chain.
    """
    clientId: str = ''
    occurrenceChain: List[ExpandPSData] = ()


@dataclass
class OccurrenceListInfo(TcBaseObj):
    """
    Input structure that contains the occurrence chains to be expanded and the property names used to construct those
    occurrence chains.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    
    :var attributeNames: List of one or more attribute names that correspond to the values that make up the
    'OccurrenceChainList'. Each attribute name identifies the BOMLine property  that was used to obtain the identifier
    in the occurrence chain at the same  position. The 'attributeNames' and the 'occurrenceChainsByParent' lists are
    parallel in that regard. However, if the 'attributeNames' list length is less than the 'occurrenceChainsByParent'
    length then it is assumed that the last name in the 'attributeNames' list is repeated out until they are of equal
    length.
    
    :var occurrenceChainsByParent: The list of occurrences to be expanded into BOMLines.
    """
    clientId: str = ''
    attributeNames: List[str] = ()
    occurrenceChainsByParent: List[OccurrenceChainList] = ()


@dataclass
class OccurrenceListResults(TcBaseObj):
    """
    This structure records the parent context and the occurrences expanded in that context.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errorsassociated with this input structure.
    
    :var parent: The parent BOMLine and any additionally requested related objects associated with the BOMLine.
    
    :var occurrenceList: List of 'OccurrenceChainResults' that describe the requested occurrences in the context of the
    given parent BOMLine.
    """
    clientId: str = ''
    parent: ExpandPSParentData = None
    occurrenceList: List[OccurrenceChainResult] = ()


@dataclass
class RelatedObjectTypeAndNamedRefs(TcBaseObj):
    """
    This structure contains a related object type and the list of its named references to be processed.
    
    :var relationTypeName: The related object type for which the specified named references will be processed.
    :var namedReferenceNames: Named references to be processed for the object types.
    """
    relationTypeName: str = ''
    namedReferenceNames: List[str] = ()


@dataclass
class RelationAndTypesFilter(TcBaseObj):
    """
    This structure contains a relation name and a list of related object types and its named references
    ('RelatedObjectTypeAndNamedReferences').  Together they are used to filter objects passed back in a product
    structure expand operation.
    
    :var relationName: Name of the relation by which the objects related to the BOMLines are to be filtered.
    :var relatedObjAndNamedRefs: A list of related object types and named references that are related that are to be
    filtered. The objects filtered will have to be related to the BOMLine by the relation of type specified by the
    'relationName' member.
    
    :var namedRefHandler: An enumeration used to identify how named references are processed.
    
    - UseNamedRefsList : The named references as specified in the 'RelatedObjectAndNamedRefs' structure will be
    processed. 
    - NoNamedRefs : No named references are to be processed. This will override the 'namedReferenceNames' specified in
    the 'RelatedObjectAndNamedRefs' structure. Even if the 'namedReferenceNames' has any values in it, no named
    references will be returned for this option.
    - AllNamedRefs : All the named references , related to the BOMLine  will be processed and sent to the client.
    - PreferredJT : The preferred JT named reference will be returned. Please refer to the  Determining JT file
    priorities section in the Administering the product structure topic of the Structure Manager Guide in Teamcenter
    documentation for more details on the preferred JT files. 
    
    """
    relationName: str = ''
    relatedObjAndNamedRefs: List[RelatedObjectTypeAndNamedRefs] = ()
    namedRefHandler: NamedRefHandler = None


@dataclass
class CreateBOMsFromRecipesInfo(TcBaseObj):
    """
    Input structure that defines the BOM recipe object and optional topline.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure. 
    
    :var recipeObj: Object of type ConfigurationContext, Fnd0StructureContext or StructureContext that records the
    configuration recipe.
    :var topline: Top line object that must be non null if recipeObj is of type ConfigurationContext. This object is
    either the BOMView or BOMViewRevision of the top line. If recipeObj is of type StructureContext or
    Fnd0StructureContext then this argument is ignored.
    """
    clientId: str = ''
    recipeObj: BusinessObject = None
    topline: BusinessObject = None


@dataclass
class CreateBOMsFromRecipesOutput(TcBaseObj):
    """
    Output structure that contains the list of created 'BOMWindow' objects.
    
    :var clientId: The unique string supplied by the caller used to match the output to the supplied input.
    :var bomWindow: The 'BOMWindow' object that was created and configured according to the input recipe. 
    If the recipe objects represent a manufacturing composition then the additional composition windows will also have
    been created and attached to this 'BOMWindow'.
    """
    clientId: str = ''
    bomWindow: BOMWindow = None


@dataclass
class CreateBOMsFromRecipesResponse(TcBaseObj):
    """
    Response structure for the 'createBOMsFromRecipes' operation.
    
    :var output: List of 'CreateBOMsFromRecipesOutput' structures containing the BOM Windows that were created and
    configured according to the input recipe objects. One 'CreateBOMsFromRecipesOutput' returned per input recipe
    object.
    
    :var serviceData: The Service Data.
    """
    output: List[CreateBOMsFromRecipesOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateVisSCsFromBOMsInfo(TcBaseObj):
    """
    Input structure used for creating VisStrucutreContext objects based on the given
    BOMWindows and specific occurrenses within those BOMs.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements 
     and partial errors associated with this input structure.
    
    :var occurrencesList: List of BOMLines representing  the occurrences to be included in the structure recipe.
    :var staticStructureFile: IMANFile reference to the PLMXML static representation of the structure. If 
     not supplied then the associated property of the VisStructureContext will not be set.
    """
    clientId: str = ''
    occurrencesList: List[BOMLine] = ()
    staticStructureFile: BusinessObject = None


@dataclass
class CreateVisSCsFromBOMsOutput(TcBaseObj):
    """
    The output structure that contains the references to the created VisStructureContext
    objects along with the corresponding clientId.
    
    :var clientId: The unique string supplied by the caller used to match the output to the supplied input.
    :var structureRecipe: VisStructureContext object that records the configuration recipe of the BOMWindow that
    contains the input BOMLines.
    """
    clientId: str = ''
    structureRecipe: VisStructureContext = None


@dataclass
class CreateVisSCsFromBOMsResponse(TcBaseObj):
    """
    Response structure for the createVisSCsFromBOMs() operation.
    
    :var output: List of 'CreateVisSCsFromBOMsOutput' structures containing the VisStructureContext object that records
    the configuration recipe of the BOMWindow from which the input BOMLines belong.
    
    :var serviceData: The Service Data.
    """
    output: List[CreateVisSCsFromBOMsOutput] = ()
    serviceData: ServiceData = None


class NamedRefHandler(Enum):
    """
    An enumeration used to indicate which name reference handler should be used.
    
    - 'UseNamedRefsList' The named references as specified in the 'RelatedObjectAndNamedRefs' structure will be
    processed. 
    - 'NoNamedRefs' No named references are to be processed. This will override the 'namedReferenceNames' specified in
    the 'RelatedObjectAndNamedRefs' structure. Even if the 'namedReferenceNames' has any values in it, no named
    references will be returned for this option.
    - 'AllNamedRefs' All the named references, related to the BOMLine will be processed and sent to the client.
    - 'PreferredJT' The preferred JT named reference will be returned. Please refer to the Determining JT file
    Priorities section in the Administering the product structure topic of the Structure Manager Guide in Teamcenter
    documentation for more details on the preferred JT files.
    
    """
    UseNamedRefsList = 'UseNamedRefsList'
    NoNamedRefs = 'NoNamedRefs'
    AllNamedRefs = 'AllNamedRefs'
    PreferredJT = 'PreferredJT'
